@@ -12,7 +12,7 @@
  * Internally the output character set is used. Other characters are
  * encoded using Unicode entities according to HTML 4.0.
  *
- * $Id: i18n.php,v 1.95 2002/02/02 02:17:04 philippe_mingo Exp $
+ * $Id: i18n.php,v 1.98.2.1 2002/05/30 07:55:05 philippe_mingo Exp $
  */
 
 /* Decodes a string to the internal encoding from the given charset */
@@ -435,11 +435,8 @@ function charset_decode_iso_8859_7 ($string) {
      * ISO-8859-7 characters from 11/04 (0xB4) to 11/06 (0xB6)
      * These are Unicode 900-902
      */
-    while (ereg("([\264-\266])", $string, $res)) {
-        $replace = '&#' . (ord($res[1])+720) . ';';
-        $string = str_replace($res[1], $replace, $string);
-    }
-
+    $string = preg_replace("/([\264-\266])/","'&#' . (ord(\\1)+720)",$string);
+    
     /* 11/07 (0xB7) Middle dot is the same in iso-8859-1 */
     $string = str_replace("\267", '&#183;', $string);
 
@@ -447,10 +444,7 @@ function charset_decode_iso_8859_7 ($string) {
      * ISO-8859-7 characters from 11/08 (0xB8) to 11/10 (0xBA)
      * These are Unicode 900-902
      */
-    while (ereg("([\270-\272])", $string, $res)) {
-        $replace = '&#' . (ord($res[1])+720) . ";";
-        $string = str_replace($res[1], $replace, $string);
-    }
+    $string = preg_replace("/([\270-\272])/","'&#' . (ord(\\1)+720)",$string);
 
     /*
      * 11/11 (0xBB) Right angle quotation mark is the same as in
@@ -459,10 +453,7 @@ function charset_decode_iso_8859_7 ($string) {
     $string = str_replace("\273", '&#187;', $string);
 
     /* And now the rest of the charset */
-    while (ereg("([\274-\376])", $string, $res)) {
-        $replace = '&#' . (ord($res[1])+720) . ';';
-        $string = str_replace($res[1], $replace, $string);
-    }
+    $string = preg_replace("/([\274-\376])/","'&#' . (ord(\\1)+720)",$string);
 
     return $string;
 }
@@ -475,7 +466,7 @@ function charset_decode_iso_8859_15 ($string) {
     // Euro sign
     $string = str_replace ("\244", '&#8364;', $string);
     // Latin capital letter S with caron
-    $string = str_replace ("\244", '&#352;', $string);
+    $string = str_replace ("\246", '&#352;', $string);
     // Latin small letter s with caron
     $string = str_replace ("\250", '&#353;', $string);
     // Latin capital letter Z with caron
@@ -492,7 +483,7 @@ function charset_decode_iso_8859_15 ($string) {
     return (charset_decode_iso_8859_1($string));
 }
 
-/* ISO-8859-15 is Cyrillic */
+/* ISO-8859-5 is Cyrillic */
 function charset_decode_iso_8859_5 ($string) {
     // Convert to KOI8-R, then return this decoded.
     $string = convert_cyr_string($string, 'i', 'k');
@@ -869,12 +860,16 @@ $languages['tr_TR']['NAME']    = 'Turkish';
 $languages['tr_TR']['CHARSET'] = 'iso-8859-9';
 $languages['tr']['ALIAS'] = 'tr_TR';
 
-// it's zh_TW, actually.
-
 $languages['zh_TW']['NAME']    = 'Taiwan';
 $languages['zh_TW']['CHARSET'] = 'big5';
 $languages['tw']['ALIAS'] = 'zh_TW';
 
+/*
+$languages['zh_TW']['NAME']    = 'Chinese';
+$languages['zh_TW']['CHARSET'] = 'gb2312';
+$languages['tw']['ALIAS'] = 'zh_CN';
+*/
+
 $languages['sk_SK']['NAME']     = 'Slovak';
 $languages['sk_SK']['CHARSET']  = 'iso-8859-2';
 $languages['sk']['ALIAS']       = 'sk_SK';
