@@ -8,7 +8,7 @@
  *
  * IMAP search routines
  *
- * $Id: imap_search.php,v 1.44 2002/02/12 14:58:09 jmunro Exp $
+ * $Id: imap_search.php,v 1.52.2.2 2002/06/13 14:44:00 jmunro Exp $
  */
 
 require_once('../functions/imap.php');
@@ -17,88 +17,96 @@ require_once('../functions/array.php');
 require_once('../functions/mailbox_display.php');
 require_once('../functions/mime.php');
 
-function sqimap_search($imapConnection,$search_where,$search_what,$mailbox,$color, $search_position = '', $search_all, $count_all) {
+function sqimap_search($imapConnection, $search_where, $search_what, $mailbox,
+                       $color, $search_position = '', $search_all, $count_all) {
 
-    global $msgs, $message_highlight_list, $squirrelmail_language, $languages, $index_order;
-    global $pos;
+    global $msgs, $message_highlight_list, $squirrelmail_language, $languages,
+           $index_order, $pos, $allow_charset_search, $imap_server_type;
 
     $pos = $search_position;
 
     $urlMailbox = urlencode($mailbox);
 
-    /*
-        Construct the Search QuERY
+    /* construct the search query, taking multiple search terms into account */
+    $multi_search = array();
+    $search_what  = trim($search_what);
+    $search_what  = ereg_replace('[ ]{2,}', ' ', $search_what);
+    $multi_search = explode(' ', $search_what);
+    $search_string = '';
 
-        account for multiple search terms
-    */
+    /* it seems macosx does not support the prefered search 
+       syntax so we fall back to the older style. This IMAP
+       server has a problem with multiple search terms. Instead
+       of returning the messages that match all the terms it
+       returns the messages that match each term. Could be fixed
+       on the client side, but should be fixed on the server
+       as per the RFC */
 
-    $multi_search = array ();
-    $search_what = ereg_replace("[ ]{2,}", ' ', $search_what);
-    $multi_search = split (' ', $search_what);
-    if (count($multi_search)==1) {
-            $search_string = $search_where . ' ' . '"' . $multi_search[0] . '"';
+    if ($imap_server_type == 'macosx') {
+        foreach ($multi_search as $multi_search_part) {
+            $search_string .= $search_where . ' ' .$multi_search_part . ' ';
+        }
     }
     else {
-            $search_string = '';
-    $count = count($multi_search);
-            for ($x=0;$x<$count;$x++) {
-			trim($multi_search[$x]);
-                $search_string = $search_string . ' ' . $search_where . ' "' . $multi_search[$x] . '"';
-            }
+        foreach ($multi_search as $multi_search_part) {
+            $search_string .= $search_where . ' {' . strlen($multi_search_part)
+                . "}\r\n" . $multi_search_part . ' ';
+        }
     }
+
     $search_string = trim($search_string);
 
-/* now use $search_string in the imap search */
-
-    if (isset($languages[$squirrelmail_language]['CHARSET']) &&
+    /* now use $search_string in the imap search */
+    if ($allow_charset_search && isset($languages[$squirrelmail_language]['CHARSET']) &&
         $languages[$squirrelmail_language]['CHARSET']) {
-        $ss = "SEARCH CHARSET ".$languages[$squirrelmail_language]['CHARSET']." ALL $search_string";
+        $ss = "SEARCH CHARSET "
+            . strtoupper($languages[$squirrelmail_language]['CHARSET']) 
+            . " ALL $search_string";
     } else {
-        $ss .= "SEARCH ALL $search_string\"";
+        $ss = "SEARCH ALL $search_string";
     }
-    /* Read Data Back From IMAP */
-    $readin = sqimap_run_command ($imapConnection, $ss, true, $result, $message);
-    if (isset($languages[$squirrelmail_language]['CHARSET']) && strtolower($result) == 'no') {
-        // $ss = "SEARCH CHARSET \"US-ASCII\" ALL $search_where \"$search_what\"";
+
+    /* read data back from IMAP */
+    $readin = sqimap_run_command($imapConnection, $ss, false, $result, $message);
+
+    /* try US-ASCII charset if search fails */
+    if (isset($languages[$squirrelmail_language]['CHARSET']) 
+        && strtolower($result) == 'no') {
         $ss = "SEARCH CHARSET \"US-ASCII\" ALL $search_string";
-        $readin = sqimap_run_command ($imapConnection, $ss, true, $result, $message);
+        $readin = sqimap_run_command ($imapConnection, $ss, true, 
+                                      $result, $message);
     }
 
     unset($messagelist);
     $msgs = '';
-    $c = 0;
 
-    /* Keep going till we find the SEARCH responce */
-    while ($c < count( $readin )) {
-
-        /* Check to see if a SEARCH Responce was recived */
-        if (substr($readin[$c],0,9) == "* SEARCH ")
-            $messagelist = explode(" ",substr($readin[$c],9));
-        else if (isset($errors))
-            $errors = $errors.$readin[$c];
-        else
-            $errors = $readin[$c];
-        $c++;
+    /* Keep going till we find the SEARCH response */
+    foreach ($readin as $readin_part) {
+        /* Check to see if a SEARCH response was received */
+        if (substr($readin_part, 0, 9) == '* SEARCH ') {
+            $messagelist = explode(' ', substr($readin_part, 9));
+        } else if (isset($errors)) {
+            $errors = $errors.$readin_part;
+        } else {
+            $errors = $readin_part;
+        }
     }
 
     /* If nothing is found * SEARCH should be the first error else echo errors */
-    if (isset($errors) && strstr($errors,"* SEARCH")) {
-        if ($search_all != "all") {
-            echo '<br><CENTER>' . _("No Messages Found") . '</CENTER>';
+    if (isset($errors)) {
+        if (strstr($errors,'* SEARCH')) {
+            if ($search_all != 'all') {
+                echo '<br><CENTER>' . _("No Messages Found") . '</CENTER>';
+            }
             return;
         }
-        else {
-            return;
-        }
-    }
-    else if (isset($errors)) {
-        echo "<!-- ".$errors." -->";
+        echo "<!-- $errors -->";
     }
 
     /*
-        HACKED CODED FROM ANOTHER FUNCTION, Could Probably dump this and mondify
-        exsitising code with a search true/false varible.
-    */
+     * HACKED CODE FROM ANOTHER FUNCTION, could probably dump this and modify
+     * existing code with a search true/false variable.
+     */
 
     global $sent_folder;
     for ($q = 0; $q < count($messagelist); $q++) {
@@ -106,7 +114,8 @@ function sqimap_search($imapConnection,$search_where,$search_what,$mailbox,$colo
     }
     $issent = ($mailbox == $sent_folder);
     $hdr_list = sqimap_get_small_header_list($imapConnection, $id, $issent);
-    $flags = sqimap_get_flags_list($imapConnection, $id, $issent);
+//    $flags = sqimap_get_flags_list($imapConnection, $id, $issent);
+
     foreach ($hdr_list as $hdr) {
         $from[] = $hdr->from;
         $date[] = $hdr->date;
@@ -116,12 +125,16 @@ function sqimap_search($imapConnection,$search_where,$search_what,$mailbox,$colo
         $cc[] = $hdr->cc;
         $size[] = $hdr->size;
         $type[] = $hdr->type0;
+	$flag_deleted[] = $hdr->flag_deleted;
+	$flag_answered[] = $hdr->flag_answered;
+	$flag_seen[] = $hdr->flag_seen;
+	$flag_flagged[] = $hdr->flag_flagged;
     }
 
     $j = 0;
     while ($j < count($messagelist)) {
             $date[$j] = str_replace('  ', ' ', $date[$j]);
-            $tmpdate = explode(" ", trim($date[$j]));
+            $tmpdate = explode(' ', trim($date[$j]));
 
             $messages[$j]["TIME_STAMP"] = getTimeStamp($tmpdate);
             $messages[$j]["DATE_STRING"] = getDateString($messages[$j]["TIME_STAMP"]);
@@ -135,7 +148,11 @@ function sqimap_search($imapConnection,$search_where,$search_what,$mailbox,$colo
             $messages[$j]["CC"] = $cc[$j];
             $messages[$j]["SIZE"] = $size[$j];
             $messages[$j]["TYPE0"] = $type[$j];
-
+    	    $messages[$j]['FLAG_DELETED'] = $flag_deleted[$j];
+            $messages[$j]['FLAG_ANSWERED'] = $flag_answered[$j];
+            $messages[$j]['FLAG_SEEN'] = $flag_seen[$j];
+            $messages[$j]['FLAG_FLAGGED'] = $flag_flagged[$j];
+/*
             $num = 0;
             while ($num < count($flags[$j])) {
                 if ($flags[$j][$num] == 'Deleted') {
@@ -149,7 +166,9 @@ function sqimap_search($imapConnection,$search_where,$search_what,$mailbox,$colo
                 }
                 $num++;
             }
+*/	    
             $j++;
+
     }
 
     /* Find and remove the ones that are deleted */
@@ -183,15 +202,15 @@ function sqimap_search($imapConnection,$search_where,$search_what,$mailbox,$colo
             }
             mail_message_listing_beginning( $imapConnection,
                 "move_messages.php?msg=$msg&mailbox=$urlMailbox&pos=$pos&where=" . urlencode($search_where) . "&what=".urlencode($search_what),
-            $mailbox,
+                $mailbox,
                 -1,
                 '<b>' . _("Found") . ' ' . count($messagelist) . ' ' . _("messages") . '</b></tr><tr>'.
-            get_selectall_link($start_msg, $sort));
+                get_selectall_link($start_msg, $sort));
         }
         else {
             mail_message_listing_beginning( $imapConnection,
                 "move_messages.php?msg=$msg&mailbox=$urlMailbox&pos=$pos&where=" . urlencode($search_where) . "&what=".urlencode($search_what),
-            $mailbox,
+                $mailbox,
                 -1,
                 '<b>' . _("Found") . ' ' . count($messagelist) . ' ' . _("messages") . '</b></tr><tr>');
         }
