@@ -9,67 +9,14 @@
  * This code provides various string manipulation functions that are
  * used by the rest of the Squirrelmail code.
  *
- * $Id: strings.php,v 1.142 2002/02/22 13:03:34 teepe Exp $
+ * $Id: strings.php,v 1.153.2.2 2002/06/21 20:21:20 teepe Exp $
  */
 
 /**
  * SquirrelMail version number -- DO NOT CHANGE
  */
 global $version;
-$version = '1.2.5';
-
-/**
- * If $haystack is a full mailbox name and $needle is the mailbox
- * separator character, returns the last part of the mailbox name.
- */
-function readShortMailboxName($haystack, $needle) {
-    if ($needle == '') {
-        return $haystack;
-    }
-    $parts = explode($needle, $haystack);
-    $elem = array_pop($parts);
-    while ($elem == '' && count($parts)) {
-        $elem = array_pop($parts);
-    }
-    return $elem;
-}
-
-/**
- * If $haystack is a full mailbox name, and $needle is the mailbox
- * separator character, returns the second last part of the full
- * mailbox name (i.e. the mailbox's parent mailbox)
- */
-function readMailboxParent($haystack, $needle) {
-    if ($needle == '') return '';
-    $parts = explode($needle, $haystack);
-    $elem = array_pop($parts);
-    while ($elem == '' && count($parts)) {
-        $elem = array_pop($parts);
-    }
-    return join($needle, $parts);
-}
-
-/**
- * Returns the index of the first chunk of string $haystack that
- * starts with non-white-space character, starting at position $pos.
- * If there is no such chunk, returns -1.
- */
-function next_pos_minus_white ($haystack, $pos) {
-    $len = strlen($haystack);
-    while ($pos < $len) {
-        /* Get the next character. */
-        $c = substr($haystack, $pos, 1);
-        
-        /* Check the next character. */
-        if (($c != ' ') && ($c != "\t") && ($c != "\n") && ($c != "\r")) {
-            return $pos;
-        }
-
-        /* Increment position in string. */
-        ++$pos;
-    }
-    return -1;
-}
+$version = '1.2.7';
 
 /**
  * Wraps text at $wrap characters
@@ -117,41 +64,28 @@ function sqWordWrap(&$line, $wrap) {
         
         /* Go to the next line if we have more to process */
         if ($i < count($words)) {
-            $line .= "\n" . $beginning_spaces;
+            $line .= "\n";
         }
     }
 }
 
-
 /**
- * Does the opposite of sqWordWrap()
+ * If $haystack is a full mailbox name and $needle is the mailbox
+ * separator character, returns the last part of the mailbox name.
  */
-function sqUnWordWrap(&$body) {
-    $lines = explode("\n", $body);
-    $body = '';
-    $PreviousSpaces = '';
-    for ($i = 0; $i < count($lines); $i ++) {
-        ereg("^([\t >]*)([^\t >].*)?$", $lines[$i], $regs);
-        $CurrentSpaces = $regs[1];
-        if (isset($regs[2])) {
-            $CurrentRest = $regs[2];
-        }
-        
-        if ($i == 0) {
-            $PreviousSpaces = $CurrentSpaces;
-            $body = $lines[$i];
-        } else if (($PreviousSpaces == $CurrentSpaces) /* Do the beginnings match */
-                   && (strlen($lines[$i - 1]) > 65)    /* Over 65 characters long */
-                   && strlen($CurrentRest)) {          /* and there's a line to continue with */
-            $body .= ' ' . $CurrentRest;
-        } else {
-            $body .= "\n" . $lines[$i];
-            $PreviousSpaces = $CurrentSpaces;
-        }
-    }
-    $body .= "\n";
-}
+function readShortMailboxName($haystack, $needle) {
 
+    if ($needle == '') {
+        $elem = $haystack;
+    } else {
+	    $parts = explode($needle, $haystack);
+	    $elem = array_pop($parts);
+	    while ($elem == '' && count($parts)) {
+	        $elem = array_pop($parts);
+	    }
+    }
+    return( $elem );
+}
 
 /**
  * Returns an array of email addresses.
@@ -189,64 +123,6 @@ function getLineOfAddrs($array) {
     return( $to_line );
 }
 
-function translateText(&$body, $wrap_at, $charset) {
-    global $where, $what; /* from searching */
-    global $color; /* color theme */
-    
-    require_once('../functions/url_parser.php');
-    
-    $body_ary = explode("\n", $body);
-    $PriorQuotes = 0;
-    for ($i=0; $i < count($body_ary); $i++) {
-        $line = $body_ary[$i];
-        if (strlen($line) - 2 >= $wrap_at) {
-            sqWordWrap($line, $wrap_at);
-        }
-        $line = charset_decode($charset, $line);
-        $line = str_replace("\t", '        ', $line);
-
-        parseUrl ($line);
-
-        $Quotes = 0;
-        $pos = 0;
-	$j = strlen( $line );
-
-        while ( $pos < $j ) {
-            if ($line[$pos] == ' ') {
-                $pos ++;
-            } else if (strpos($line, '&gt;', $pos) === $pos) {
-                $pos += 4;
-                $Quotes ++;
-            } else {
-                break;
-            }
-        }
-        
-        if ($Quotes > 1) {
-            if (! isset($color[14])) {
-                $color[14] = '#FF0000';
-            }
-            $line = '<FONT COLOR="' . $color[14] . '">' . $line . '</FONT>';
-        } elseif ($Quotes) {
-            if (! isset($color[13])) {
-                $color[13] = '#800000';
-            }
-            $line = '<FONT COLOR="' . $color[13] . '">' . $line . '</FONT>';
-        }
-        
-        $body_ary[$i] = $line;
-    }
-    $body = '<pre>' . implode("\n", $body_ary) . '</pre>';
-}
-
-function find_mailbox_name ($mailbox) {
-    if (ereg(" *\"([^\r\n\"]*)\"[ \r\n]*$", $mailbox, $regs))
-        return $regs[1];
-    ereg(" *([^ \r\n\"]*)[ \r\n]*$",$mailbox,$regs);
-    return $regs[1];
-    
-}
-
 function php_self () {
     global $PHP_SELF, $HTTP_SERVER_VARS;
     
@@ -277,7 +153,7 @@ function php_self () {
 function get_location () {
     
     global $PHP_SELF, $SERVER_NAME, $HTTP_HOST, $SERVER_PORT,
-        $HTTP_SERVER_VARS;
+        $HTTP_SERVER_VARS, $imap_server_type;
     
     /* Get the path, handle virtual directories */
     $path = substr(php_self(), 0, strrpos(php_self(), '/'));
@@ -319,6 +195,15 @@ function get_location () {
             }
         }
     }
+
+    /* this is a workaround for the weird macosx caching that
+       causes Apache to return 16080 as the port number, which causes
+       SM to bail */
+           
+    if ($imap_server_type == 'macosx' && $port == ':16080') {
+        $port = '';
+    }
+    
     
     /* Fallback is to omit the server name and use a relative */
     /* URI, although this is not RFC 2616 compliant.          */
