@@ -8,7 +8,7 @@
  *
  * Philippe Mingo
  *
- * $Id: defines.php,v 1.19 2002/02/13 15:35:14 simond Exp $
+ * $Id: defines.php,v 1.23 2002/05/01 15:20:00 jmunro Exp $
  */
 
 require_once( '../functions/constants.php' );
@@ -63,6 +63,14 @@ $defcfg = array( '$config_version' => array( 'name' => _("Config File Version"),
                  '$org_logo' => array( 'name' => _("Organization Logo"),
                                        'type' => SMOPT_TYPE_STRING,
                                        'size' => 40 ),
+                 '$org_logo_width' => array( 'name'    => _("Organization Logo Width"),
+                                             'type'    => SMOPT_TYPE_INTEGER,
+                                             'size'    => 5,
+                                             'default' => 0),
+                 '$org_logo_height' => array( 'name'    => _("Organization Logo Height"),
+                                              'type'    => SMOPT_TYPE_INTEGER,
+                                              'size'    => 5,
+                                              'default' => 0),
                  '$org_title' => array( 'name' => _("Organization Title"),
                                         'type' => SMOPT_TYPE_STRING,
                                         'size' => 40 ),
@@ -114,6 +122,8 @@ $defcfg = array( '$config_version' => array( 'name' => _("Config File Version"),
                                                      'type' => SMOPT_TYPE_BOOLEAN ),
                  '$invert_time' => array( 'name' => _("Invert Time"),
                                           'type' => SMOPT_TYPE_BOOLEAN ),
+                 '$default_use_mdn' => array( 'name' => _("Use Confirmation Flags"),
+                                              'type' => SMOPT_TYPE_BOOLEAN ),                                          
                  /* --------------------------------------------------------*/
                  'Group3' => array( 'name' => _("Folders Defaults"),
                                     'type' => SMOPT_TYPE_TITLE ),
@@ -167,11 +177,11 @@ $defcfg = array( '$config_version' => array( 'name' => _("Config File Version"),
                                     'type' => SMOPT_TYPE_TITLE ),
                  '$default_charset' => array( 'name' => _("Default Charset"),
                                               'type' => SMOPT_TYPE_STRLIST,
-                                              'posvals' => array( 'iso_8859_1' => 'iso_8859_1',
-                                                                  'iso_8859_2' => 'iso_8859_2',
-                                                                  'iso_8859_7' => 'iso_8859_7',
-                                                                  'iso_8859_15' => 'iso_8859_15',
-                                                                  'iso_8859_15' => 'iso_8859_15',
+                                              'posvals' => array( 'iso-8859-1' => 'iso-8859-1',
+                                                                  'iso-8859-2' => 'iso-8859-2',
+                                                                  'iso-8859-7' => 'iso-8859-7',
+                                                                  'iso-8859-15' => 'iso-8859-15',
+                                                                  'iso-8859-15' => 'iso-8859-15',
                                                                   'ns_4551_1' => 'ns_4551_1',
                                                                   'koi8-r' => 'koi8-r',
                                                                   'euc-KR' => 'euc-KR',
@@ -227,6 +237,18 @@ $defcfg = array( '$config_version' => array( 'name' => _("Config File Version"),
                                           'type' => SMOPT_TYPE_STRING,
                                           'size' => 40,
                                           'default' => 'userprefs' ),
+                 '$prefs_user_field' => array('name' => _("Preferences username field"),
+                                              'type' => SMOPT_TYPE_STRING,
+                                              'size' => 40,
+                                              'default' => 'user' ),
+                 '$prefs_key_field' => array('name' => _("Preferences key field"),
+                                             'type' => SMOPT_TYPE_STRING,
+                                             'size' => 40,
+                                             'default' => 'prefkey' ),
+                 '$prefs_val_field' => array('name' => _("Preferences value field"),
+                                             'type' => SMOPT_TYPE_STRING,
+                                             'size' => 40,
+                                             'default' => 'prefval' ),
                  /* --------------------------------------------------------*/
                  'Group7' => array( 'name' => _("Themes"),
                                     'type' => SMOPT_TYPE_TITLE ),
