@@ -14,7 +14,7 @@
  *       addrbook_search.html -- If you change one, change
  *       the other one too!
  *
- * $Id: addrbook_search_html.php,v 1.44 2002/02/11 18:25:59 jmunro Exp $
+ * $Id: addrbook_search_html.php,v 1.47 2002/05/15 15:17:22 stekkel Exp $
  */
 
 require_once('../src/validate.php');
@@ -28,13 +28,14 @@ require_once('../functions/strings.php');
 /* Insert hidden data */
 function addr_insert_hidden() {
     global $body, $subject, $send_to, $send_to_cc, $send_to_bcc, $mailbox,
-           $identity;
+           $identity, $session;
 
    echo '<input type=hidden value="';
    if (substr($body, 0, 1) == "\r") {
        echo "\n";
    }
    echo htmlspecialchars($body) . '" name=body>' . "\n" .
+        '<input type=hidden value="' . $session . '" name=session>' . "\n" .
         '<input type=hidden value="' . htmlspecialchars($subject) .
         '" name=subject>' . "\n" .
         '<input type=hidden value="' . htmlspecialchars($send_to) .
@@ -53,23 +54,45 @@ function addr_insert_hidden() {
 
 /* List search results */
 function addr_display_result($res, $includesource = true) {
-    global $color, $PHP_SELF;
+    global $color, $javascript_on, $PHP_SELF;
 
     if (sizeof($res) <= 0) return;
 
-    echo '<form method=post action="' . $PHP_SELF . "\">\n" .
+    echo '<form method=post action="' . $PHP_SELF . '" name="addrbook">'."\n" .
          '<input type=hidden name="html_addr_search_done" value="true">' . "\n";
     addr_insert_hidden();
     $line = 0;
 
-    echo '<TABLE BORDER=0 WIDTH="98%" ALIGN=center>' .
-         '<TR BGCOLOR="' . $color[9] . '"><TH ALIGN=left>&nbsp;' .
-         '<TH ALIGN=left>&nbsp;' . _("Name") .
-         '<TH ALIGN=left>&nbsp;' . _("E-mail") .
-         '<TH ALIGN=left>&nbsp;' . _("Info");
+if ($javascript_on) {
+    print
+        '<script language="JavaScript" type="text/javascript">' .
+        "\n<!-- \n" .
+        "function CheckAll(ch) {\n" .
+        "   for (var i = 0; i < document.addrbook.elements.length; i++) {\n" .
+        "       if( document.addrbook.elements[i].type == 'checkbox' &&\n" .
+        "           document.addrbook.elements[i].name.substr(0,16) == 'send_to_search['+ch ) {\n" .
+        "           document.addrbook.elements[i].checked = !(document.addrbook.elements[i].checked);\n".
+        "       }\n" .
+        "   }\n" .
+        "}\n" .
+        "//-->\n" .
+        "</script>\n";
+    $chk_all = '<a href="#" onClick="CheckAll(\'T\');">' . _("All") . '</a>&nbsp;<font color="'.$color[9].'">To</font>'.
+            '&nbsp;&nbsp;'.
+            '<a href="#" onClick="CheckAll(\'C\');">' . _("All") . '</a>&nbsp;<font color="'.$color[9].'">Cc</font>'.
+            '&nbsp;&nbsp;'.
+            '<a href="#" onClick="CheckAll(\'B\');">' . _("All") . '</a>';
+    }
+    echo '<TABLE BORDER="0" WIDTH="98%" ALIGN="center">' .
+           '<TR BGCOLOR="' . $color[9] . '"><TH ALIGN="left">&nbsp;' . $chk_all . '</TH>' .
+          '<TH ALIGN="left">&nbsp;' . _("Name") . '</TH>' .
+          '<TH ALIGN="left">&nbsp;' . _("E-mail") . '</TH>' .
+          '<TH ALIGN="left">&nbsp;' . _("Info") . '</TH>';
+
+
 
     if ($includesource) {
-        echo '<TH ALIGN=left WIDTH="10%">&nbsp;' . _("Source");
+        echo '<TH ALIGN=left WIDTH="10%">&nbsp;' . _("Source"). '</TH>';
     }
 
     echo "</TR>\n";
@@ -94,7 +117,7 @@ function addr_display_result($res, $includesource = true) {
          $line ++;
     }
     echo '<TR><TD ALIGN=center COLSPAN=';
-    if ($includesource) { echo '4'; } else { echo '5'; }
+    if ($includesource) { echo '5'; } else { echo '4'; }
     echo '><INPUT TYPE=submit NAME="addr_search_done" VALUE="' .
          _("Use Addresses") . '"></TD></TR>' .
          '</TABLE>' .
@@ -152,6 +175,10 @@ if ($abook->numbackends > 1) {
 } else {
     echo '<INPUT TYPE=hidden NAME=backend VALUE=-1>' . "\n";
 }
+if (isset($session)) {
+    echo "<input type=hidden name=\"session\" value=\"$session\">";
+}
+
 echo '<INPUT TYPE=submit VALUE="' . _("Search") . '">' .
      '&nbsp;|&nbsp;<INPUT TYPE=submit VALUE="' . _("List all") .
      '" NAME=listall>' . "\n" .
