@@ -8,7 +8,7 @@
  *
  * Manage personal address book.
  *
- * $Id: addressbook.php,v 1.42 2002/02/11 18:25:59 jmunro Exp $
+ * $Id: addressbook.php,v 1.43.2.1 2002/06/12 14:38:16 jmunro Exp $
  */
 
 require_once('../src/validate.php');
@@ -264,7 +264,7 @@ if ($showaddrlist) {
     
             /* New table header for each backend */
             if($prevbackend != $row['backend']) {
-                if($prevbackend >= 0) {
+                if($prevbackend < 0) {
                     echo '<TR><TD COLSPAN=5 ALIGN=center>' . "\n" .
                          '<INPUT TYPE=submit NAME=editaddr VALUE="' . 
                          _("Edit selected") . "\">\n" .
@@ -309,12 +309,14 @@ if ($showaddrlist) {
                  '&nbsp;</TD>' .
                  '<TD VALIGN=top NOWRAP WIDTH="1%">&nbsp;' . $row['name'] .
                  '&nbsp;</TD>',
-                 '<TD VALIGN=top NOWRAP WIDTH="1%">&nbsp;' .
-                 '<A HREF="compose.php?send_to=' . rawurlencode($row['email']);
+                 '<TD VALIGN=top NOWRAP WIDTH="1%">&nbsp;'; 
                 if ($compose_new_win == '1') {
-                     echo '" TARGET="compose_window" onClick="comp_in_new()"';
+                    echo '<a href="javascript:void(0)" onclick=comp_in_new(false,"compose.php?send_to='.rawurlencode($row['email']).'")>';
                 }
-                echo '">' . $row['email'] . '</A>&nbsp;</TD>'."\n",
+                else {
+                 echo '<A HREF="compose.php?send_to=' . rawurlencode($row['email']).'">';
+                }
+                echo $row['email'] . '</A>&nbsp;</TD>'."\n",
                  '<TD VALIGN=top WIDTH="1%">&nbsp;' . $row['label'] . '&nbsp;</TD>' .
                  "</TR>\n";
             $line++;
