@@ -14,7 +14,7 @@
  *    - Send mail
  *    - Save As Draft
  *
- * $Id: compose.php,v 1.204 2002/02/20 09:48:07 simond Exp $
+ * $Id: compose.php,v 1.225.2.1 2002/06/21 01:37:58 jmunro Exp $
  */
 
 require_once('../src/validate.php');
@@ -25,11 +25,56 @@ require_once('../functions/smtp.php');
 require_once('../functions/display_messages.php');
 require_once('../functions/plugin.php');
 
+/* --------------------- Specific Functions ------------------------------ */
+
+
+
+/**
+ * Does the opposite of sqWordWrap()
+ */
+function sqUnWordWrap(&$body) {
+    $lines = explode("\n", $body);
+    $body = '';
+    $PreviousSpaces = '';
+    for ($i = 0; $i < count($lines); $i ++) {
+        ereg("^([\t >]*)([^\t >].*)?$", $lines[$i], $regs);
+        $CurrentSpaces = $regs[1];
+        if (isset($regs[2])) {
+            $CurrentRest = $regs[2];
+        }
+        
+        if ($i == 0) {
+            $PreviousSpaces = $CurrentSpaces;
+            $body = $lines[$i];
+        } else if (($PreviousSpaces == $CurrentSpaces) /* Do the beginnings match */
+                   && (strlen($lines[$i - 1]) > 65)    /* Over 65 characters long */
+                   && strlen($CurrentRest)) {          /* and there's a line to continue with */
+            $body .= ' ' . $CurrentRest;
+        } else {
+            $body .= "\n" . $lines[$i];
+            $PreviousSpaces = $CurrentSpaces;
+        }
+    }
+    $body .= "\n";
+}
+
+/* ----------------------------------------------------------------------- */
+
 if (!isset($attachments)) {
     $attachments = array();
     session_register('attachments');
 }
 
+if (!isset($composesession)) {
+    $composesession = 0;
+    session_register('composesession');
+}
+
+if (!isset($session) || (isset($newmessage) && $newmessage)) {
+    $session = "$composesession" +1; 
+    $composesession = $session;        
+}     
+
 if (!isset($mailbox) || $mailbox == '' || ($mailbox == 'None')) {
     $mailbox = 'INBOX';
 }
@@ -42,8 +87,11 @@ if (isset($draft)) {
     if (! isset($MDN)) {
         $MDN = 'False';
     }
-    if (!saveMessageAsDraft($send_to, $send_to_cc, $send_to_bcc, $subject, $body, $reply_id, $MDN)) {
-        showInputForm();
+    if (! isset($mailprio)) {
+        $mailprio = '';
+    }
+    if (!saveMessageAsDraft($send_to, $send_to_cc, $send_to_bcc, $subject, $body, $reply_id, $mailprio, $session)) {
+        showInputForm($session);
         exit();
     } else {
         $draft_message = _("Draft Email Saved");
@@ -55,7 +103,7 @@ if (isset($draft)) {
         }
         else {
             if ($compose_new_win == '1') {
-                Header("Location: compose.php?saved_draft=yes");
+                Header("Location: compose.php?saved_draft=yes&session=$composesession");
             exit();
             }
             else {
@@ -71,7 +119,7 @@ if (isset($send)) {
     if (isset($HTTP_POST_FILES['attachfile']) &&
         $HTTP_POST_FILES['attachfile']['tmp_name'] &&
         $HTTP_POST_FILES['attachfile']['tmp_name'] != 'none') {
-        $AttachFailure = saveAttachedFiles();
+        $AttachFailure = saveAttachedFiles($session);
     }
     if (checkInput(false) && !isset($AttachFailure)) {
         $urlMailbox = urlencode (trim($mailbox));
@@ -94,8 +142,7 @@ if (isset($send)) {
         /*
          * Rewrap $body so that no line is bigger than $editor_size
          * This should only really kick in the sqWordWrap function
-         * if the browser doesn't support "HARD" as the wrap type
-         * Or, in Opera's case, something goes wrong.
+         * if the browser doesn't support "VIRTUAL" as the wrap type.
          */
         $body = explode("\n", $body);
         $newBody = '';
@@ -106,8 +153,8 @@ if (isset($send)) {
             if (strlen($line) <= $editor_size + 1) {
                 $newBody .= $line . "\n";
             } else {
-                sqWordWrap($line, $editor_size) . "\n";
-                $newBody .= $line;
+                sqWordWrap($line, $editor_size);
+                $newBody .= $line . "\n";
             }
         }
         $body = $newBody;
@@ -117,13 +164,13 @@ if (isset($send)) {
         $MDN = False;  // we are not sending a mdn response
         if (! isset($mailprio)) {
             $Result = sendMessage($send_to, $send_to_cc, $send_to_bcc,
-                                  $subject, $body, $reply_id, $MDN);
+                                  $subject, $body, $reply_id, $MDN, '', $session);
         } else {
             $Result = sendMessage($send_to, $send_to_cc, $send_to_bcc,
-                                  $subject, $body, $reply_id, $MDN, $mailprio);
+                                  $subject, $body, $reply_id, $MDN, $mailprio, $session);
         }
         if (! $Result) {
-            showInputForm();
+            showInputForm($session);
             exit();
         }
         if ( isset($delete_draft)) {
@@ -155,7 +202,7 @@ if (isset($send)) {
         }
 
         checkInput(true);
-        showInputForm();
+        showInputForm($session);
         /* sqimap_logout($imapConnection); */
     }
 } elseif (isset($html_addr_search_done)) {
@@ -188,12 +235,12 @@ if (isset($send)) {
             }
         }
     }
-    showInputForm();
+    showInputForm($session);
 } elseif (isset($html_addr_search)) {
     if (isset($HTTP_POST_FILES['attachfile']) &&
         $HTTP_POST_FILES['attachfile']['tmp_name'] &&
         $HTTP_POST_FILES['attachfile']['tmp_name'] != 'none') {
-        if (saveAttachedFiles()) {
+        if (saveAttachedFiles($session)) {
             plain_error_message(_("Could not move/copy file. File not attached"), $color);
         }
     }
@@ -203,7 +250,7 @@ if (isset($send)) {
      */
     include_once('./addrbook_search_html.php');
 } elseif (isset($attach)) {
-    if (saveAttachedFiles()) {
+    if (saveAttachedFiles($session)) {
         plain_error_message(_("Could not move/copy file. File not attached"), $color);
     }
         if ($compose_new_win == '1') {
@@ -212,7 +259,7 @@ if (isset($send)) {
         else {
             displayPageHeader($color, $mailbox);
         }
-    showInputForm();
+    showInputForm($session);
 }
 elseif (isset($sigappend)) {
     $idents = getPref($data_dir, $username, 'identities', 0);
@@ -230,7 +277,7 @@ elseif (isset($sigappend)) {
     } else {
         displayPageHeader($color, $mailbox);
     }
-    showInputForm();
+    showInputForm($session);
 } elseif (isset($do_delete)) {
         if ($compose_new_win == '1') {
             compose_Header($color, $mailbox);
@@ -244,12 +291,33 @@ elseif (isset($sigappend)) {
         foreach($delete as $index) {
             $attached_file = $hashed_attachment_dir . '/'
                            . $attachments[$index]['localfilename'];
-            unlink ($attached_file);
-            unset ($attachments[$index]);
+    	    unlink ($attached_file);
+    	    unset ($attachments[$index]);
         }
     }
 
-    showInputForm();
+    showInputForm($session);
+    
+} elseif (isset($attachedmessages)) {
+
+    /*
+     * This handles the case if we attache message 
+     */
+    $imapConnection = sqimap_login($username, $key, $imapServerAddress,
+                                   $imapPort, 0);
+        if ($compose_new_win == '1') {
+            compose_Header($color, $mailbox);
+        }
+        else {
+            displayPageHeader($color, $mailbox);
+        }
+
+    $newmail = true;
+
+    newMail();
+    showInputForm($session);
+    sqimap_logout($imapConnection);
+
 } else {
     /*
      * This handles the default case as well as the error case
@@ -266,18 +334,22 @@ elseif (isset($sigappend)) {
 
     $newmail = true;
 
-    ClearAttachments();
+    ClearAttachments($session);
 
     if (isset($forward_id) && $forward_id && isset($ent_num) && $ent_num) {
-        getAttachments(0);
+        getAttachments(0, $session);
     }
 
     if (isset($draft_id) && $draft_id && isset($ent_num) && $ent_num) {
-        getAttachments(0);
+        getAttachments(0, $session);
     }
 
-    newMail();
-    showInputForm();
+    if (isset($passed_id) && $passed_id && isset($ent_num) && $ent_num) {
+        getAttachments(0, $session);
+    }
+
+    newMail($session);
+    showInputForm($session);
     sqimap_logout($imapConnection);
 }
 
@@ -291,18 +363,25 @@ exit();
 function newMail () {
     global $forward_id, $imapConnection, $msg, $ent_num, $body_ary, $body,
            $reply_id, $send_to, $send_to_cc, $mailbox, $send_to_bcc, $editor_size,
-           $draft_id, $use_signature;
+           $draft_id, $use_signature, $composesession, $forward_cc, $passed_id,
+           $edit_as_new;
 
     $send_to = decodeHeader($send_to, false);
     $send_to_cc = decodeHeader($send_to_cc, false);
     $send_to_bcc = decodeHeader($send_to_bcc, false);
+    $send_to = str_replace('&lt;', '<', str_replace('&gt;', '>', str_replace('&amp;', '&', str_replace('&quot;', '"', $send_to))));
+    $send_to_cc = str_replace('&lt;', '<', str_replace('&gt;', '>', str_replace('&amp;', '&', str_replace('&quot;', '"', $send_to_cc))));
+    $send_to_bcc = str_replace('&lt;', '<', str_replace('&gt;', '>', str_replace('&amp;', '&', str_replace('&quot;', '"', $send_to_bcc))));
 
     if ($forward_id) {
         $id = $forward_id;
     } elseif ($reply_id) {
         $id = $reply_id;
+    } elseif ($passed_id) {
+        $id = $passed_id;
     }
 
+
     if ($draft_id){
         $id = $draft_id;
         $use_signature = FALSE;
@@ -335,6 +414,21 @@ function newMail () {
         }
 
         sqUnWordWrap($body);
+        
+        /* this corrects some wrapping/quoting problems on replies */
+        if ($reply_id || $edit_as_new || $forward_id) {
+            $rewrap_body = explode("\n", $body);
+            for ($i=0;$i<count($rewrap_body);$i++) {
+                sqWordWrap($rewrap_body[$i], ($editor_size - 2));
+                if (preg_match("/^(>+)/", $rewrap_body[$i], $matches)) {
+                    $gt = $matches[1];
+                    $rewrap_body[$i] = str_replace("\n", "\n$gt ", $rewrap_body[$i]);
+                }
+                $rewrap_body[$i] .= "\n";
+            }
+            $body = implode("", $rewrap_body);
+        }
+
         $body_ary = explode("\n", $body);
         $i = count($body_ary) - 1;
         while ($i >= 0 && ereg("^[>\\s]*$", $body_ary[$i])) {
@@ -344,14 +438,14 @@ function newMail () {
         $body = '';
         for ($i=0; isset($body_ary[$i]); $i++) {
             if ($reply_id) {
-                if (ereg('^[ >]+', $body_ary[$i])) {
+                if (preg_match("/^(>){1,}/", $body_ary[$i])) {
                     $body_ary[$i] = '>' . $body_ary[$i];
                 } else {
                     $body_ary[$i] = '> ' . $body_ary[$i];
                 }
             }
-            if (!$draft_id) {
-                sqWordWrap($body_ary[$i], $editor_size - 1);
+            if ($draft_id) {
+                sqWordWrap($body_ary[$i], $editor_size );
             }
             $body .= $body_ary[$i] . "\n";
             unset($body_ary[$i]);
@@ -368,6 +462,14 @@ function newMail () {
                     $bodyTop .= '         ' . $orig_header->to[$x] . "\n";
                 }
             }
+            if (isset($forward_cc) && $forward_cc) {
+                $bodyTop .= _("Cc") . ': ' . $orig_header->cc[0] . "\n";
+                if (count($orig_header->cc) > 1) {
+                    for ($x = 1; $x < count($orig_header->cc); $x++) {
+                        $bodyTop .= '         ' . $orig_header->cc[$x] . "\n";
+                    }
+                }
+            }
             $bodyTop .= "\n";
             $body = $bodyTop . $body;
         }
@@ -414,14 +516,16 @@ function newMail () {
 } /* function newMail() */
 
 
-function getAttachments($message) {
+function getAttachments($message, $session) {
     global $mailbox, $attachments, $attachment_dir, $imapConnection,
-           $ent_num, $forward_id, $draft_id, $username;
+           $ent_num, $forward_id, $draft_id, $username, $passed_id;
 
     if (isset($draft_id)) {
         $id = $draft_id;
-    } else {
+    } else if (isset($forward_id)) {
         $id = $forward_id;
+    } else {
+        $id = $passed_id;
     }
 
     if (!$message) {
@@ -450,6 +554,8 @@ function getAttachments($message) {
             $newAttachment['remotefilename'] = $filename;
             $newAttachment['type'] = strtolower($message->header->type0 .
                                                 '/' . $message->header->type1);
+	    $newAttachment['id'] = strtolower($message->header->id);
+	    $newAttachment['session'] = $session;
 
             /* Write Attachment to file */
             $fp = fopen ("$hashed_attachment_dir/$localfilename", 'w');
@@ -462,13 +568,13 @@ function getAttachments($message) {
         }
     } else {
         for ($i = 0; $i < count($message->entities); $i++) {
-            getAttachments($message->entities[$i]);
+            getAttachments($message->entities[$i], $session);
         }
     }
     return;
 }
 
-function showInputForm () {
+function showInputForm ($session) {
     global $send_to, $send_to_cc, $reply_subj, $forward_subj, $body,
            $passed_body, $color, $use_signature, $signature, $prefix_sig,
            $editor_size, $attachments, $subject, $newmail,
@@ -476,7 +582,7 @@ function showInputForm () {
            $from_htmladdr_search, $location_of_buttons, $attachment_dir,
            $username, $data_dir, $identity, $draft_id, $delete_draft,
            $mailprio, $default_use_mdn, $mdn_user_support, $compose_new_win,
-           $saved_draft, $mail_sent;
+           $saved_draft, $mail_sent, $sig_first, $edit_as_new;
 
     $subject = decodeHeader($subject, false);
     $reply_subj = decodeHeader($reply_subj, false);
@@ -500,12 +606,16 @@ function showInputForm () {
     
     echo ">\n";
 
-    if (isset($draft_id)) {
+    if (isset($draft_id) && !$edit_as_new) {
         echo '<input type="hidden" name="delete_draft" value="' . $draft_id . "\">\n";
     }
     if (isset($delete_draft)) {
         echo '<input type="hidden" name="delete_draft" value="' . $delete_draft. "\">\n";
     }
+    if (isset($session)) {
+        echo '<input type="hidden" name="session" value="' . "$session" . "\">\n";
+    }
+
     if ($saved_draft == 'yes') {
         echo '<BR><CENTER><B>'. _("Draft Saved").'</CENTER></B>';
     }
@@ -514,6 +624,7 @@ function showInputForm () {
     }
     echo '<TABLE WIDTH="100%" ALIGN=center CELLSPACING=0 BORDER=0>' . "\n";
     if ($compose_new_win == '1') {
+        echo '<TABLE ALIGN=CENTER BGCOLOR="'.$color[0].'" WIDTH="100%" BORDER=0>'."\n";
         echo '   <TR><TD></TD><TD ALIGN="RIGHT"><INPUT TYPE="BUTTON" NAME="Close" onClick="return self.close()" VALUE='._("Close").'></TD></TR>'."\n";
     }
     if ($location_of_buttons == 'top') {
@@ -605,13 +716,30 @@ function showInputForm () {
     if ($location_of_buttons == 'between') {
         showComposeButtonRow();
     }
-
-    echo '   <TR>' . "\n" .
-         '      <TD BGCOLOR="' . $color[4] . '" COLSPAN=2>' . "\n" .
-         '         &nbsp;&nbsp;<TEXTAREA NAME=body ROWS=20 COLS="' .
-         $editor_size . '" WRAP=HARD>' . htmlspecialchars($body);
+    if ($compose_new_win == '1') {
+        echo '   <TR>' . "\n" .
+             '      <TD BGCOLOR="' . $color[0] . '" COLSPAN=2 ALIGN=CENTER>' . "\n" .
+             '         <TEXTAREA NAME=body ROWS=20 COLS="' .
+             $editor_size . '" WRAP="VIRTUAL">';
+    }
+    else {
+        echo '   <TR>' . "\n" .
+            '      <TD BGCOLOR="' . $color[4] . '" COLSPAN=2>' . "\n" .
+            '         &nbsp;&nbsp;<TEXTAREA NAME=body ROWS=20 COLS="' .
+            $editor_size . '" WRAP="VIRTUAL">';
+    }
     if ($use_signature == true && $newmail == true && !isset($from_htmladdr_search)) {
-        echo "\n\n".($prefix_sig==true? "-- \n":'').htmlspecialchars($signature);
+        if ($sig_first == '1') {
+            echo "\n\n".($prefix_sig==true? "-- \n":'').htmlspecialchars($signature);
+            echo "\n\n".htmlspecialchars($body);
+        }
+        else {
+            echo "\n\n".htmlspecialchars($body);
+            echo "\n\n".($prefix_sig==true? "-- \n":'').htmlspecialchars($signature);
+        }
+    }
+    else {
+       echo htmlspecialchars($body);
     }
     echo '</TEXTAREA><BR>' . "\n" .
          '      </TD>' . "\n" .
@@ -642,10 +770,12 @@ function showInputForm () {
              '&nbsp;' .
              '</td><td align=left bgcolor="' . $color[0] . '">';
         foreach ($attachments as $key => $info) {
-            $attached_file = "$hashed_attachment_dir/$info[localfilename]";
-            echo '<input type="checkbox" name="delete[]" value="' . $key . "\">\n" .
-                 $info['remotefilename'] . ' - ' . $info['type'] . ' (' .
-                 show_readable_size(filesize($attached_file)) . ")<br>\n";
+    	    if ($info['session'] == $session) { 
+            	$attached_file = "$hashed_attachment_dir/$info[localfilename]";
+            	echo '<input type="checkbox" name="delete[]" value="' . $key . "\">\n" .
+                        $info['remotefilename'] . ' - ' . $info['type'] . ' (' .
+                        show_readable_size( filesize( $attached_file ) ) . ")<br>\n";
+    	    }
         }
 
         echo '<input type="submit" name="do_delete" value="' .
@@ -653,7 +783,9 @@ function showInputForm () {
              '</td></tr>';
     }
     /* End of attachment code */
-
+    if ($compose_new_win == '1') {
+        echo '</TABLE>'."\n";
+    }
     echo '</TABLE>' . "\n";
     if ($reply_id) {
         echo '<input type=hidden name=reply_id value=' . $reply_id . ">\n";
@@ -669,6 +801,7 @@ function showInputForm () {
 function showComposeButtonRow() {
     global $use_javascript_addr_book, $save_as_draft,
         $default_use_priority, $mailprio, $default_use_mdn,
+	$request_mdn, $request_dr,
         $data_dir, $username;
 
     echo "  <TR><TD>\n</TD><TD>\n";
@@ -686,8 +819,10 @@ function showComposeButtonRow() {
     if ($default_use_mdn) {
         if ($mdn_user_support) {
             echo "\n\t". _("Receipt") .': '.
-            '<input type="checkbox" name="request_mdn" value=1>'. _("On read").
-            ' <input type="checkbox" name="request_dr" value=1>'. _("On Delivery");
+            '<input type="checkbox" name="request_mdn" value=1'.
+		($request_mdn=='1'?' checked':'') .'>'. _("On read").
+            ' <input type="checkbox" name="request_dr" value=1'.
+		($request_dr=='1'?' checked':'') .'>'. _("On Delivery");
         }
     }
 
@@ -736,7 +871,7 @@ function checkInput ($show) {
 
 
 /* True if FAILURE */
-function saveAttachedFiles() {
+function saveAttachedFiles($session) {
     global $HTTP_POST_FILES, $attachment_dir, $attachments, $username;
 
     $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
@@ -756,6 +891,7 @@ function saveAttachedFiles() {
     $newAttachment['localfilename'] = $localfilename;
     $newAttachment['remotefilename'] = $HTTP_POST_FILES['attachfile']['name'];
     $newAttachment['type'] = strtolower($HTTP_POST_FILES['attachfile']['type']);
+    $newAttachment['session'] = $session;
 
     if ($newAttachment['type'] == "") {
          $newAttachment['type'] = 'application/octet-stream';
@@ -765,23 +901,29 @@ function saveAttachedFiles() {
 }
 
 
-
-
-function ClearAttachments()
+function ClearAttachments($session)
 {
     global $username, $attachments, $attachment_dir;
     $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
 
-    foreach ($attachments as $info) {
-        $attached_file = "$hashed_attachment_dir/$info[localfilename]";
-        if (file_exists($attached_file)) {
-            unlink($attached_file);
+    $rem_attachments = array();
+    if (is_array($attachments)) {
+        foreach ($attachments as $info) {
+	        if ($info['session'] == $session) {
+    	        $attached_file = "$hashed_attachment_dir/$info[localfilename]";
+    	        if (file_exists($attached_file)) {
+        	        unlink($attached_file);
+    	        }
+	        } 
+            else {
+	            $rem_attachments[] = $info;
+	        }
         }
     }
-
-    $attachments = array();
+    $attachments = $rem_attachments;
 }
 
+
 function getReplyCitation($orig_from)
 {
     global $reply_citation_style, $reply_citation_start, $reply_citation_end;
@@ -819,7 +961,7 @@ function getReplyCitation($orig_from)
         $end   = '">';
         break;
     case 'user-defined':
-        $start = $reply_citation_start;
+        $start = $reply_citation_start . ' ';
         $end   = $reply_citation_end;
         break;
     default:
