@@ -9,7 +9,7 @@
  * This is the code for the left bar. The left bar shows the folders
  * available, and has cookie information.
  *
- * $Id: left_main.php,v 1.123 2002/02/19 01:58:47 brong Exp $
+ * $Id: left_main.php,v 1.131.2.1 2002/05/28 22:53:57 jmunro Exp $
  */
 
 require_once('../src/validate.php');
@@ -65,7 +65,7 @@ function formatMailboxName($imapConnection, $box_array) {
     if ($unseen > 0) { $line .= '<B>'; }
 
     /* Crate the link for this folder. */
-    $line .= "<A HREF=\"right_main.php?sort=0&amp;startMessage=1&amp;mailbox=$mailboxURL\" TARGET=\"right\" STYLE=\"text-decoration:none\">";
+    $line .= "<A HREF=\"right_main.php?PG_SHOWALL=0&amp;sort=0&amp;startMessage=1&amp;mailbox=$mailboxURL\" TARGET=\"right\" STYLE=\"text-decoration:none\">";
     if ($special_color) {
         $line .= "<FONT COLOR=\"$color[11]\">";
     }
@@ -94,7 +94,7 @@ function formatMailboxName($imapConnection, $box_array) {
         if ($numMessages > 0) {
             $urlMailbox = urlencode($real_box);
             $line .= "\n<small>\n" .
-                    "&nbsp;&nbsp;(<A HREF=\"empty_trash.php\" style=\"text-decoration:none\">"._("empty")."</A>)" .
+                    "&nbsp;&nbsp;(<A HREF=\"empty_trash.php\" style=\"text-decoration:none\">"._("purge")."</A>)" .
                     "</small>";
         }
     }
@@ -192,16 +192,18 @@ function is_parent_box($curbox_name, $parbox_name) {
     return ($parbox_name == $actual_parname);
 }
 
-
 /* -------------------- MAIN ------------------------ */
 
-global $delimiter, $default_folder_prefix;
+global $delimiter, $default_folder_prefix, $left_size;
 
 // open a connection on the imap port (143)
 $imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 10); // the 10 is to hide the output
 
-
-if (isset($left_refresh) && ($left_refresh != 'none') && ($left_refresh != '')) {
+/**
+ * Using stristr since older preferences may contain "None" and "none".
+ */
+if (isset($left_refresh) && ($left_refresh != '') &&
+    !stristr($left_refresh, "none")){
     $xtra =  "\n<META HTTP-EQUIV=\"Expires\" CONTENT=\"Thu, 01 Dec 1994 16:00:00 GMT\">\n" .
              "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n".
              "<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"$left_refresh;URL=left_main.php\">\n";
@@ -233,8 +235,6 @@ echo "\n<BODY BGCOLOR=\"$color[3]\" TEXT=\"$color[6]\" LINK=\"$color[6]\" VLINK=
 
 do_hook('left_main_before');
 
-$boxes = sqimap_mailbox_list($imapConnection);
-
 echo '<CENTER><FONT SIZE=4><B>'. _("Folders") . "</B><BR></FONT>\n\n";
 
 if ($date_format != 6) {
@@ -286,6 +286,8 @@ if ( $collapse_folders ) {
     }
 }
 
+ 
+$boxes = sqimap_mailbox_list($imapConnection);
 /* Prepare do do out collapsedness and visibility computation. */
 $curbox = 0;
 $boxcount = count($boxes);
@@ -340,10 +342,9 @@ for ($i = 0; $i < count($boxes); $i++) {
         echo $line;
     }
 }
-
-sqimap_logout($imapConnection);
 do_hook('left_main_after');
+sqimap_logout($imapConnection);
 
-echo "</BODY></HTML>\n";
+echo "</div></BODY></HTML>\n";
 
 ?>
