@@ -9,7 +9,7 @@
  * This a simple login screen. Some housekeeping is done to clean
  * cookies and find language.
  *
- * $Id: login.php,v 1.70 2002/02/19 12:31:12 brong Exp $
+ * $Id: login.php,v 1.76 2002/05/16 08:18:53 stekkel Exp $
  */
 
 $rcptaddress = '';
@@ -66,9 +66,13 @@ require_once('../functions/page_header.php');
  */
 set_up_language($squirrelmail_language, TRUE);
 
-/* Need the base URI to set the cookies. (Same code as in webmail.php). */
-ereg ("(^.*/)[^/]+/[^/]+$", $PHP_SELF, $regs);
-$base_uri = $regs[1];
+/**
+ * Find out the base URI to set cookies.
+ */
+if (!function_exists('sqm_baseuri')){
+    require_once('../functions/display_messages.php');
+}
+$base_uri = sqm_baseuri();
 @session_destroy();
 
 /*
@@ -76,7 +80,8 @@ $base_uri = $regs[1];
  * we get a new one.
  */
 $cookie_params = session_get_cookie_params();
-setcookie(session_name(),'',0,$cookie_params['path'].$cookie_params['domain']);
+setcookie(session_name(), '', 0, $cookie_params['path'], 
+          $cookie_params['domain']);
 setcookie('username', '', 0, $base_uri);
 setcookie('key', '', 0, $base_uri);
 header('Pragma: no-cache');
@@ -85,7 +90,7 @@ do_hook('login_cookie');
 
 /* Output the javascript onload function. */
 
-$header = "<SCRIPT LANGUAGE=\"JavaScript\">\n" .
+$header = "<SCRIPT LANGUAGE=\"JavaScript\" TYPE=\"text/javascript\">\n" .
           "<!--\n".
           "  function squirrelmail_loginpage_onload() {\n".
           "    document.forms[0].js_autodetect_results.value = '" . SMPREF_JS_ON . "';\n".
@@ -97,8 +102,8 @@ $custom_css = 'none';
 displayHtmlHeader( "$org_name - " . _("Login"), $header, FALSE );
 
 /* Set the title of this page. */
-echo "<BODY TEXT=\"#000000\" BGCOLOR=\"#FFFFFF\" LINK=\"#0000CC\" VLINK=\"#0000CC\" ALINK=\"#0000CC\" onLoad='squirrelmail_loginpage_onload();'>\n".
-     "<FORM ACTION=\"redirect.php\" METHOD=\"POST\" NAME=f>\n";
+echo '<BODY TEXT="#000000" BGCOLOR="#FFFFFF" LINK="#0000CC" VLINK="#0000CC" ALINK="#0000CC" onLoad="squirrelmail_loginpage_onload();">'.
+     "\n<FORM ACTION=\"redirect.php\" METHOD=\"POST\">\n";
 
 $username_form_name = 'login_username';
 $password_form_name = 'secretkey';
@@ -106,15 +111,25 @@ do_hook('login_top');
 
 $loginname_value = (isset($loginname) ? htmlspecialchars($loginname) : '');
 
-echo "<CENTER>".
-     "  <IMG SRC=\"$org_logo\"><BR>\n".
+/* Display width and height like good little people */
+$width_and_height = '';
+if (isset($org_logo_width) && is_int($org_logo_width) && $org_logo_width>0) {
+    $width_and_height = " WIDTH=\"$org_logo_width\"";
+}
+if (isset($org_logo_height) && is_int($org_logo_height) && $org_logo_height>0) {
+    $width_and_height .= " HEIGHT=\"$org_logo_height\"";
+}
+
+echo '<CENTER>'.
+     "  <IMG SRC=\"$org_logo\" ALT=\"" . sprintf(_("%s Logo"), $org_name) . 
+        "\"$width_and_height><BR>\n".
      ( $hide_sm_attributions ? '' :
        '<SMALL>' . sprintf (_("SquirrelMail version %s"), $version) . "<BR>\n".
        '  ' . _("By the SquirrelMail Development Team") . "<BR></SMALL>\n" ) .
      "</CENTER>\n".
 
      "<CENTER>\n".
-     "<TABLE COLS=1 WIDTH=350>\n".
+     "<TABLE COLS=\"1\" WIDTH=\"350\">\n".
      "   <TR><TD ALIGN=CENTER BGCOLOR=\"#DCDCDC\">\n".
      '      <B>' . sprintf (_("%s Login"), $org_name) . "</B>\n".
      "   </TD></TR>".
@@ -149,4 +164,4 @@ echo "</FORM>\n";
 do_hook('login_bottom');
 echo "</BODY>\n".
      "</HTML>\n";
-?>
+?>
\ No newline at end of file
