@@ -8,7 +8,7 @@
  *
  * Displays all options relating to personal information
  *
- * $Id: options_personal.php,v 1.34 2002/02/19 23:12:09 teepe Exp $
+ * $Id: options_personal.php,v 1.36 2002/05/05 11:11:33 ullgren Exp $
  */
 
 require_once('../functions/imap.php');
@@ -116,29 +116,31 @@ function load_optpage_data_personal() {
             'comment' =>  $identities_link_value
         );
     }
-
-    $TZ_ARRAY[SMPREF_NONE] = _("Same as server");
-    $fd = fopen('../locale/timezones.cfg','r');
-    while (!feof ($fd)) {
-        $zone = fgets($fd, 1024);
-        if( $zone ) {
-	    $zone = trim($zone);
-            $TZ_ARRAY["$zone"] = "$zone";
-        }
-    }
-    fclose ($fd);
     
-    $optgrps[SMOPT_GRP_TZ] = _("Timezone Options");
-    $optvals[SMOPT_GRP_TZ] = array();
+    if (  !ini_get("safe_mode") ) {
+        $TZ_ARRAY[SMPREF_NONE] = _("Same as server");
+        $fd = fopen('../locale/timezones.cfg','r');
+        while (!feof ($fd)) {
+            $zone = fgets($fd, 1024);
+            if( $zone ) {
+                $zone = trim($zone);
+                $TZ_ARRAY["$zone"] = "$zone";
+            }
+        }
+        fclose ($fd);
 
-    $optvals[SMOPT_GRP_TZ][] = array(
-        'name'    => 'timezone',
-	'caption' => _("Your current timezone"),
-	'type'    => SMOPT_TYPE_STRLIST,
-	'refresh' => SMOPT_REFRESH_NONE,
-	'posvals' => $TZ_ARRAY
-    );
-											     
+        $optgrps[SMOPT_GRP_TZ] = _("Timezone Options");
+        $optvals[SMOPT_GRP_TZ] = array();
+
+        $optvals[SMOPT_GRP_TZ][] = array(
+            'name'    => 'timezone',
+            'caption' => _("Your current timezone"),
+            'type'    => SMOPT_TYPE_STRLIST,
+            'refresh' => SMOPT_REFRESH_NONE,
+            'posvals' => $TZ_ARRAY
+        );
+    }
+    
     /*** Load the Reply Citation Options into the array ***/
     $optgrps[SMOPT_GRP_REPLY] = _("Reply Citation Options");
     $optvals[SMOPT_GRP_REPLY] = array();
