@@ -11,7 +11,7 @@
  *
  * - this is the page that does all the work, really.
  *
- * $Id: printer_friendly_bottom.php,v 1.8 2002/01/28 18:33:33 kink Exp $
+ * $Id: printer_friendly_bottom.php,v 1.9.2.1 2002/06/21 02:00:00 graf25 Exp $
  */
 
 require_once('../src/validate.php');
@@ -90,15 +90,15 @@ if ( empty($pf_cleandisplay) || $pf_cleandisplay != 'no' ) {
 displayHtmlHeader( _("Printer Friendly"), '', FALSE );
 
 echo "<body text=\"$color[8]\" bgcolor=\"$color[4]\" link=\"$color[7]\" vlink=\"$color[7]\" alink=\"$color[7]\">\n" .
-     /* headers (we use table becasue translations are not all the same width) */
+     /* headers (we use table because translations are not all the same width) */
      '<table>'.
-     '<tr><td>' . _("From") . ':</td><td>' . htmlentities($from) . "</td></td>\n".
-     '<tr><td>' . _("To") . ':</td><td>' . htmlentities($to) . "</td></td>\n";
+     '<tr><td>' . _("From") . ':</td><td>' . htmlspecialchars($from) . "</td></tr>\n".
+     '<tr><td>' . _("To") . ':</td><td>' . htmlspecialchars($to) . "</td></tr>\n";
 if ( strlen($cc) > 0 ) { /* only show CC: if it's there... */
-     echo '<tr><td>' . _("CC") . ':</td><td>' . htmlentities($cc) . "</td></td>\n";
+     echo '<tr><td>' . _("CC") . ':</td><td>' . htmlspecialchars($cc) . "</td></tr>\n";
 }
-echo '<tr><td>' . _("Date") . ':</td><td>' . htmlentities($date) . "</td></td>\n".
-     '<tr><td>' . _("Subject") . ':</td><td>' . htmlentities($subject) . "</td></td>\n".
+echo '<tr><td>' . _("Date") . ':</td><td>' . htmlspecialchars($date) . "</td></tr>\n".
+     '<tr><td>' . _("Subject") . ':</td><td>' . htmlspecialchars($subject) . "</td></tr>\n".
      '</table>'.
      "\n";
 /* body */
@@ -107,7 +107,7 @@ echo $body;
 
 /* --end browser output-- */
 
-echo '</pre></body></html>';
+echo '</body></html>';
 
 /* --start pf-specific functions-- */
 
@@ -147,4 +147,4 @@ function pf_clean_string ( $unclean_string, $num_leading_spaces ) {
 
 /* --end pf-specific functions */
 
-?>
\ No newline at end of file
+?>
