@@ -9,7 +9,7 @@
  * This file is used for reading the msgs array and displaying
  * the resulting emails in the right frame.
  *
- * $Id: read_body.php,v 1.154 2002/02/19 19:31:48 philippe_mingo Exp $
+ * $Id: read_body.php,v 1.179.2.3 2002/06/10 18:16:32 jmunro Exp $
  */
 
 require_once('../src/validate.php');
@@ -20,35 +20,65 @@ require_once('../functions/url_parser.php');
 require_once('../functions/smtp.php');
 
 /**
-* Given an IMAP message id number, this will look it up in the cached
-* and sorted msgs array and return the index. Used for finding the next
-* and previous messages.
-*
-* returns the index of the next valid message from the array
-*/
+ * Given an IMAP message id number, this will look it up in the cached
+ * and sorted msgs array and return the index. Used for finding the next
+ * and previous messages.
+ *
+ * @return the index of the next valid message from the array
+ */
 function findNextMessage() {
-    global $msort, $currentArrayIndex, $msgs, $sort;
+    global $msort, $currentArrayIndex, $msgs, $sort, 
+           $thread_sort_messages, $allow_server_sort,
+           $server_sort_array;
+    if (!is_array($server_sort_array)) {
+        $thread_sort_messages = 0;
+        $allow_server_sort = FALSE;
+    }
     $result = -1;
-
-    if ($sort == 6) {
+    if ($thread_sort_messages == 1 || $allow_server_sort == TRUE) {
+        reset($server_sort_array);
+        while(list($key, $value) = each ($server_sort_array)) {
+            if ($currentArrayIndex == $value) {
+                if ($key == (count($server_sort_array) - 1)) {
+                    $result = -1;
+                    break;
+                }
+                $result = $server_sort_array[$key + 1];
+                break; 
+            }
+        }
+    } 
+    elseif ($sort == 6 && $allow_server_sort != TRUE &&
+            $thread_sort_messages != 1) {
         if ($currentArrayIndex != 1) {
             $result = $currentArrayIndex - 1;
         }
-    } else {
+    }
+    elseif ($allow_server_sort != TRUE && $thread_sort_messages != 1 ) {
+        if (!is_array($msort)) {
+            return -1;
+        }
         for (reset($msort); ($key = key($msort)), (isset($key)); next($msort)) {
             if ($currentArrayIndex == $msgs[$key]['ID']) {
                 next($msort);
                 $key = key($msort);
-                if (isset($key))
+                if (isset($key)){
                     $result = $msgs[$key]['ID'];
                     break;
+                }
             }
         }
     }
     return ($result);
 }
 
-/** Removes just one address from the list of addresses. */
+/**
+ * Removes just one address from the list of addresses. 
+ * 
+ * @param  &$addr_list  a by-ref array of addresses
+ * @param  $addr        an address to remove
+ * @return              void, since it operates on a by-ref param
+ */
 function RemoveAddress(&$addr_list, $addr) {
     if ($addr != '') {
         foreach (array_keys($addr_list, $addr) as $key_to_delete) {
@@ -60,16 +90,37 @@ function RemoveAddress(&$addr_list, $addr) {
 /** returns the index of the previous message from the array. */
 function findPreviousMessage() {
     global $msort, $currentArrayIndex, $sort, $msgs, $imapConnection,
-           $mailbox, $data_dir, $username;
-
+           $mailbox, $data_dir, $username, $thread_sort_messages,
+           $allow_server_sort, $server_sort_array;
     $result = -1;
-
-    if ($sort == 6) {
+    if (!is_array($server_sort_array)) {
+        $thread_sort_messages = 0;
+        $allow_server_sort = FALSE;
+    }
+    if ($thread_sort_messages == 1 || $allow_server_sort == TRUE) {
+        reset($server_sort_array);
+        while(list($key, $value) = each ($server_sort_array)) {
+            if ($currentArrayIndex == $value) {
+                if ($key == 0) {
+                    $result = -1;
+                    break;
+                }
+                $result = $server_sort_array[$key -1];
+                break;
+            }
+        }
+    }
+    elseif ($sort == 6 && $allow_server_sort != TRUE && 
+            $thread_sort_messages != 1) {
         $numMessages = sqimap_get_num_messages($imapConnection, $mailbox);
         if ($currentArrayIndex != $numMessages) {
             $result = $currentArrayIndex + 1;
         }
-    } else {
+    } 
+    elseif ($thread_sort_messages != 1 && $allow_server_sort != TRUE) {
+        if (!is_array($msort)) {
+            return -1;
+        }
         for (reset($msort); ($key = key($msort)), (isset($key)); next($msort)) {
             if ($currentArrayIndex == $msgs[$key]['ID']) {
                 prev($msort);
@@ -85,9 +136,9 @@ function findPreviousMessage() {
 }
 
 /**
-* Displays a link to a page where the message is displayed more
-* "printer friendly".
-*/
+ * Displays a link to a page where the message is displayed more
+ * "printer friendly".
+ */
 function printer_friendly_link() {
     global $passed_id, $mailbox, $ent_num, $color,
            $pf_subtle_link,
@@ -98,17 +149,17 @@ function printer_friendly_link() {
     }
 
     $params = '?passed_ent_id=' . $ent_num .
-              '&mailbox=' . urlencode($mailbox) .
-              '&passed_id=' . $passed_id;
+        '&mailbox=' . urlencode($mailbox) .
+        '&passed_id=' . $passed_id;
 
     $print_text = _("View Printable Version");
 
     if (!$pf_subtle_link) {
         /* The link is large, on the bottom of the header panel. */
-        $result =       '<tr bgcolor="' . $color[0] . '">' .
-                        '<td class="medText" align="right" valign="top">' .
-                          '&nbsp;' .
-                        '</td><td class="medText" valign="top" colspan="2">'."\n";
+        $result = '<tr bgcolor="' . $color[0] . '">' .
+            '<td class="medText" align="right" valign="top">' .
+            '&nbsp;' .
+            '</td><td class="medText" valign="top" colspan="2">'."\n";
     } else {
         /* The link is subtle, below "view full header". */
         $result = "<BR>\n";
@@ -163,6 +214,8 @@ function SendMDN ( $recipient , $sender) {
     $subject = $header->subject;
     $now = getLongDateString( time() );
 
+    set_my_charset();
+
     $body = _("Your message") . "\r\n\r\n" .
             "\t" . _("To:") . ' ' . $to . "\r\n" .
             "\t" . _("Subject:") . ' ' . $subject . "\r\n" .
@@ -195,19 +248,19 @@ function SendMDN ( $recipient , $sender) {
     $newAttachment = array();
     $newAttachment['localfilename'] = $localfilename;
     $newAttachment['type'] = "message/disposition-notification";
-
+    $newAttachment['session']=-1;
     $attachments[] = $newAttachment;
     $MDN_to = trim($recipient);
     $reply_id = 0;
 
-    return (SendMessage($MDN_to,'','', _("Read:") . ' ' . $subject, $body,$reply_id, True, 3) );
+    return (SendMessage($MDN_to, '', '', _("Read:") . ' ' . $subject, 
+                        $body, $reply_id, True, 3, -1) );
 }
 
 
 function ToggleMDNflag ( $set ) {
     global $imapConnection, $passed_id, $mailbox;
     sqimap_mailbox_select($imapConnection, $mailbox);
-    
     $sg =  $set?'+':'-';
     $cmd = 'STORE ' . $passed_id . ' ' . $sg . 'FLAGS ($MDNSent)';
     $read = sqimap_run_command ($imapConnection, $cmd, true, $response, 
@@ -219,16 +272,111 @@ function ClearAttachments() {
 
         $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
 
+	$rem_attachments = array();
         foreach ($attachments as $info) {
-            $attached_file = "$hashed_attachment_dir/$info[localfilename]";
-            if (file_exists($attached_file)) {
-                unlink($attached_file);
-            }
-        }
-
-        $attachments = array();
+	    if ($info['session'] == -1) {
+        	$attached_file = "$hashed_attachment_dir/$info[localfilename]";
+        	if (file_exists($attached_file)) {
+            	    unlink($attached_file);
+        	}
+    	    } else {
+		$rem_attachments[] = $info;
+	    }
+	}
+        $attachments = $rem_attachments;
 }
 
+function formatRecipientString($recipients, $item ) {
+    global $base_uri, $passed_id, $urlMailbox, $startMessage, $show_more_cc, 
+           $echo_more, $echo_less, $show_more, $show_more_bcc, $sort;
+
+    $i = 0;
+    $url_string = '';
+    
+    if (isset ($recipients[0]) && trim($recipients[0])) {
+        $string = '';
+        $ary = $recipients;
+
+        switch ($item) {
+	    case 'to':
+            $show = "&amp;show_more=1&amp;show_more_cc=$show_more_cc&amp;".
+                "show_more_bcc=$show_more_bcc";
+            $show_n = "&amp;show_more=0&amp;show_more_cc=$show_more_cc&amp;". 
+                "show_more_bcc=$show_more_bcc";
+            break;
+	    case 'cc':
+            $show = "&amp;show_more=$show_more&amp;show_more_cc=1&amp;".
+                "show_more_bcc=$show_more_bcc";
+            $show_n = "&amp;show_more=$show_more&amp;show_more_cc=0&amp;".
+                "show_more_bcc=$show_more_bcc";
+            $show_more = $show_more_cc;
+            break;
+	    case 'bcc':
+            $show = "&amp;show_more=$show_more&amp;show_more_cc=$show_more_cc".
+                "&amp;show_more_bcc=1";
+            $show_n = "&amp;show_more=$show_more&amp;show_more_cc=".
+                "$show_more_cc&amp;show_more_bcc=0";
+            $show_more = $show_more_bcc;
+            break;
+	    default:
+            $break;
+        }
+        
+        while ($i < count($ary)) {
+    	    $ary[$i] = decodeHeader(htmlspecialchars($ary[$i]));
+    	    $url_string .= $ary[$i];
+    	    if ($string) {
+                $string = "$string<BR>$ary[$i]";
+    	    } else {
+                $string = "$ary[$i]";
+    	    }
+
+    	    $i++;
+    	    if (count($ary) > 1) {
+                if ($show_more == false) {
+            	    if ($i == 1) {
+                        /* From a search... */
+                        $string .= '&nbsp;(<A HREF="' . $base_uri .
+                            "src/read_body.php?mailbox=$urlMailbox&amp;".
+                            "passed_id=$passed_id&amp;";
+                        if (isset($where) && isset($what)) {
+                    	    $string .= 'what=' . urlencode($what).
+                                "&amp;where=".urlencode($where).
+                                "$show\">$echo_more</A>)";
+                        } else {
+                    	    $string .= "sort=$sort&amp;startMessage=".
+                                "$startMessage"."$show\">$echo_more</A>)";
+                        }
+                        $i = count($ary);
+            	    }
+                } else if ($i == 1) {
+            	    /* From a search... */
+            	    $string .= '&nbsp;(<A HREF="' . $base_uri .
+                        "src/read_body.php?mailbox=$urlMailbox&amp;".
+                        "passed_id=$passed_id&amp;";
+            	    if (isset($where) && isset($what)) {
+                        $string .= 'what=' . urlencode($what).
+                            "&amp;where=".urlencode($where).
+                            "$show_n\">$echo_less</A>)";
+            	    } else {
+                        $string .= "sort=$sort&amp;startMessage=$startMessage".
+                            "$show_n\">$echo_less</A>)";
+            	    }
+                }
+    	    }
+        }
+    }
+    else {
+        $string = '';
+    }
+    $url_string = urlencode($url_string);
+    $result = array();
+    $result['str'] = $string;
+    $result['url_str'] = $url_string;
+    return $result;
+}
+
+
 
 /*
  *   Main of read_boby.php  --------------------------------------------------
@@ -240,10 +388,11 @@ function ClearAttachments() {
     $passed_id
 */
 
-if ( isset( $mailbox ) ) {
+if (isset($mailbox)){
     $mailbox = urldecode( $mailbox );
 }
-$imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
+$imapConnection = sqimap_login($username, $key, $imapServerAddress, 
+                               $imapPort, 0);
 $read = sqimap_mailbox_select($imapConnection, $mailbox, false, false, true);
 
 do_hook('html_top');
@@ -251,9 +400,10 @@ do_hook('html_top');
 /*
  * The following code sets necesarry stuff for the MDN thing
  */
-if( $default_use_mdn &&
-    ( $mdn_user_support = getPref($data_dir, $username, 'mdn_user_support', $default_use_mdn) ) ) {
-
+if($default_use_mdn &&
+   ($mdn_user_support = getPref($data_dir, $username, 'mdn_user_support', 
+                                $default_use_mdn))) {
+    
     $supportMDN = ServerMDNSupport($read["PERMANENTFLAGS"]);
     $flags = sqimap_get_flags ($imapConnection, $passed_id);
     $FirstTimeSee = !(in_array( 'Seen', $flags ));
@@ -266,22 +416,29 @@ displayPageHeader($color, $mailbox);
  * The following code shows the header of the message and then exit
  */
 if (isset($view_hdr)) {
-    $read = sqimap_run_command ($imapConnection, "FETCH $passed_id BODY[HEADER]", true, $a, $b);
-
+    $read=sqimap_run_command ($imapConnection, "FETCH $passed_id BODY[HEADER]", 
+                              true, $a, $b);
+    
     echo '<BR>' .
-        '<TABLE WIDTH="100%" CELLPADDING="2" CELLSPACING="0" BORDER="0" ALIGN="CENTER">' . "\n" .
-        "   <TR><TD BGCOLOR=\"$color[9]\" WIDTH=\"100%\" ALIGN=\"CENTER\"><B>" . _("Viewing Full Header") . '</B> - '.
-        '<a href="' . $base_uri . "src/read_body.php?mailbox=".urlencode($mailbox);
+        '<TABLE WIDTH="100%" CELLPADDING="2" CELLSPACING="0" BORDER="0"'.
+        ' ALIGN="CENTER">' . "\n" .
+        "   <TR><TD BGCOLOR=\"$color[9]\" WIDTH=\"100%\" ALIGN=\"CENTER\"><B>".
+        _("Viewing Full Header") . '</B> - '.
+        '<a href="' . $base_uri . 'src/read_body.php?mailbox='.
+        urlencode($mailbox);
     if (isset($where) && isset($what)) {
         // Got here from a search
-        echo "&amp;passed_id=$passed_id&amp;where=".urlencode($where)."&amp;what=".urlencode($what).'">';
+        echo "&amp;passed_id=$passed_id&amp;where=".urlencode($where).
+            "&amp;what=".urlencode($what).'">';
     } else {
-        echo "&amp;passed_id=$passed_id&amp;startMessage=$startMessage&amp;show_more=$show_more\">";
+        echo "&amp;passed_id=$passed_id&amp;startMessage=$startMessage".
+            "&amp;show_more=$show_more\">";
     }
-    echo _("View message") . "</a></b></td></tr></table>\n" .
-         "<table width=\"99%\" cellpadding=2 cellspacing=0 border=0 align=center>\n" .
-         '<tr><td>';
-
+    echo _("View message") . "</a></b></td></tr></table>\n".
+        "<table width='99%' cellpadding='2' cellspacing='0' border='0'".
+        "align=center>\n".
+        '<tr><td>';
+    
     $cnum = 0;
     for ($i=1; $i < count($read); $i++) {
         $line = htmlspecialchars($read[$i]);
@@ -315,11 +472,11 @@ if (isset($view_hdr)) {
         }
         parseEmail($s);
         if (isset($f)) {
-        	echo "<nobr><tt><b>$f</b>$s</tt></nobr>";
+            echo "<nobr><tt><b>$f</b>$s</tt></nobr>";
         }
     }
     echo "</td></tr></table>\n" .
-         '</body></html>';
+        '</body></html>';
     sqimap_logout($imapConnection);
     exit;
 }
@@ -336,8 +493,10 @@ for ($i = 0; $i < count($msgs); $i++) {
     }
 }
 
-// $message contains all information about the message
-// including header and body
+/**
+ * $message contains all information about the message
+ * including header and body
+ */
 $message = sqimap_get_message($imapConnection, $passed_id, $mailbox);
 
 /** translate the subject and mailbox into url-able text **/
@@ -348,30 +507,40 @@ if (isset($message->header->replyto)) {
     $url_replyto = urlencode($message->header->replyto);
 }
 
-$url_replytoall   = $url_replyto;
+$url_replytoall = $url_replyto;
 
-// If we are replying to all, then find all other addresses and
-// add them to the list.  Remove duplicates.
-// This is somewhat messy, so I'll explain:
-// 1) Take all addresses (from, to, cc) (avoid nasty join errors here)
+/**
+ * If we are replying to all, then find all other addresses and
+ * add them to the list.  Remove duplicates.
+ * This is somewhat messy, so I'll explain:
+ * 1) Take all addresses (from, to, cc) (avoid nasty join errors here)
+ */
 $url_replytoall_extra_addrs = array_merge(
-    array($message->header->from),
-    $message->header->to,
-    $message->header->cc
-);
+                                          array($message->header->from),
+                                          $message->header->to,
+                                          $message->header->cc
+                                          );
 
-// 2) Make one big string out of them
+/**
+ * 2) Make one big string out of them
+ */
 $url_replytoall_extra_addrs = join(';', $url_replytoall_extra_addrs);
 
-// 3) Parse that into an array of addresses
+/**
+ * 3) Parse that into an array of addresses
+ */
 $url_replytoall_extra_addrs = parseAddrs($url_replytoall_extra_addrs);
 
-// 4) Make them unique -- weed out duplicates
-// (Coded for PHP 4.0.0)
+/**
+ * 4) Make them unique -- weed out duplicates
+ * (Coded for PHP 4.0.0)
+ */
 $url_replytoall_extra_addrs =
     array_keys(array_flip($url_replytoall_extra_addrs));
 
-// 5) Remove the addresses we'll be sending the message 'to'
+/**
+ * 5) Remove the addresses we'll be sending the message 'to'
+ */
 $url_replytoall_avoid_addrs = '';
 if (isset($message->header->replyto)) {
     $url_replytoall_avoid_addrs = $message->header->replyto;
@@ -382,29 +551,38 @@ foreach ($url_replytoall_avoid_addrs as $addr) {
     RemoveAddress($url_replytoall_extra_addrs, $addr);
 }
 
-// 6) Remove our identities from the CC list (they still can be in the
-// TO list) only if $include_self_reply_all is turned off
+/**
+ * 6) Remove our identities from the CC list (they still can be in the
+ * TO list) only if $include_self_reply_all is turned off
+ */
 if (!$include_self_reply_all) {
     RemoveAddress($url_replytoall_extra_addrs,
-                getPref($data_dir, $username, 'email_address'));
+                  getPref($data_dir, $username, 'email_address'));
     $idents = getPref($data_dir, $username, 'identities');
     if ($idents != '' && $idents > 1) {
         for ($i = 1; $i < $idents; $i ++) {
-            $cur_email_address = getPref($data_dir, $username, 'email_address' . $i);
+            $cur_email_address = getPref($data_dir, $username, 
+                                         'email_address' . $i);
             RemoveAddress($url_replytoall_extra_addrs, $cur_email_address);
         }
     }
 }
 
-// 7) Smoosh back into one nice line
+/**
+ * 7) Smoosh back into one nice line
+ */
 $url_replytoallcc = getLineOfAddrs($url_replytoall_extra_addrs);
 
-// 8) urlencode() it
+/**
+ * 8) urlencode() it
+ */
 $url_replytoallcc = urlencode($url_replytoallcc);
 
 $dateString = getLongDateString($message->header->date);
 
-// What do we reply to -- text only, if possible
+/**
+ * What do we reply to -- text only, if possible
+ */
 $ent_num = findDisplayEntity($message);
 
 /** TEXT STRINGS DEFINITIONS **/
@@ -415,133 +593,27 @@ if (!isset($show_more_cc)) {
     $show_more_cc = FALSE;
 }
 
-/** FORMAT THE TO STRING **/
-$i = 0;
-$to_string = '';
-$to_ary = $message->header->to;
-while ($i < count($to_ary)) {
-    $to_ary[$i] = htmlspecialchars(decodeHeader($to_ary[$i]));
-
-    if ($to_string) {
-        $to_string = "$to_string<BR>$to_ary[$i]";
-    } else {
-        $to_string = "$to_ary[$i]";
-    }
-
-    $i++;
-    if (count($to_ary) > 1) {
-        if ($show_more == false) {
-            if ($i == 1) {
-                /* From a search... */
-                $to_string .= '&nbsp;(<A HREF="' . $base_uri .
-                             "src/read_body.php?mailbox=$urlMailbox&amp;passed_id=$passed_id&amp;";
-                if (isset($where) && isset($what)) {
-                    $to_string .= 'where='.urlencode($where)."&amp;what=".urlencode($what)."&amp;show_more=1&amp;show_more_cc=$show_more_cc\">$echo_more</A>)";
-                } else {
-                    $to_string .= "sort=$sort&amp;startMessage=$startMessage&amp;show_more=1&amp;show_more_cc=$show_more_cc\">$echo_more</A>)";
-                }
-                $i = count($to_ary);
-            }
-        } else if ($i == 1) {
-            /* From a search... */
-            $to_string .= '&nbsp;(<A HREF="' . $base_uri .
-                         "src/read_body.php?mailbox=$urlMailbox&amp;passed_id=$passed_id&amp;";
-            if (isset($where) && isset($what)) {
-                $to_string .= 'where='.urlencode($where)."&amp;what=".urlencode($what)."&amp;show_more=0&amp;show_more_cc=$show_more_cc\">$echo_less</A>)";
-            } else {
-                $to_string .= "sort=$sort&amp;startMessage=$startMessage&amp;show_more=0&amp;show_more_cc=$show_more_cc\">$echo_less</A>)";
-            }
-        }
-    }
+if (!isset($show_more_bcc)) {
+    $show_more_bcc = FALSE;
 }
 
+/** FORMAT THE TO STRING **/
+$to = formatRecipientString($message->header->to, "to");
+$to_string = $to['str'];
+$url_to_string = $to['url_str'];
+
+
 /** FORMAT THE CC STRING **/
-$i = 0;
-if (isset ($message->header->cc[0]) && trim($message->header->cc[0])) {
-    $cc_string = '';
-    $cc_ary = $message->header->cc;
-    while ($i < count(decodeHeader($cc_ary))) {
-        $cc_ary[$i] = htmlspecialchars($cc_ary[$i]);
-        if ($cc_string) {
-            $cc_string = "$cc_string<BR>$cc_ary[$i]";
-        } else {
-            $cc_string = "$cc_ary[$i]";
-        }
 
-        $i++;
-        if (count($cc_ary) > 1) {
-            if ($show_more_cc == false) {
-                if ($i == 1) {
-                    /* From a search... */
-                    $cc_string .= '&nbsp;(<A HREF="' . $base_uri .
-                                  "src/read_body.php?mailbox=$urlMailbox&passed_id=$passed_id";
-                    if (isset($where) && isset($what)) {
-                        $cc_string .= '&amp;what='.urlencode($what)."&amp;where=".urlencode($where)."&amp;show_more_cc=1&amp;show_more=$show_more\">$echo_more</A>)";
-                    } else {
-                        $cc_string .= "&amp;sort=$sort&amp;startMessage=$startMessage&amp;show_more_cc=1&amp;show_more=$show_more\">$echo_more</A>)";
-                    }
-                    $i = count($cc_ary);
-                }
-            } else if ($i == 1) {
-                /* From a search... */
-                $cc_string .= '&nbsp;(<A HREF="' . $base_uri .
-                              "src/read_body.php?mailbox=$urlMailbox&amp;passed_id=$passed_id&amp;";
-                if (isset($where) && isset($what)) {
-                    $cc_string .= 'what=' . urlencode($what)."&amp;where=".urlencode($where)."&amp;show_more_cc=0&amp;show_more=$show_more\">$echo_less</A>)";
-                } else {
-                    $cc_string .= "sort=$sort&amp;startMessage=$startMessage&amp;show_more_cc=0&amp;show_more=$show_more\">$echo_less</A>)";
-                }
-            }
-        }
-    }
-}
-else {
-    $cc_string = '';
-}
+$cc = formatRecipientString($message->header->cc, "cc");
+$cc_string = $cc['str'];
+$url_cc_string = $cc['url_str'];
 
 /** FORMAT THE BCC STRING **/
-$i = 0;
-if (isset ($message->header->bcc[0]) && trim($message->header->bcc[0])){
-    $bcc_string = '';
-    $bcc_ary = $message->header->bcc;
-    while ($i < count(decodeHeader($bcc_ary))) {
-        $bcc_ary[$i] = htmlspecialchars($bcc_ary[$i]);
-        if ($bcc_string) {
-            $bcc_string = "$bcc_string<BR>$bcc_ary[$i]";
-        } else {
-            $bcc_string = "$bcc_ary[$i]";
-        }
 
-        $i++;
-        if (count($bcc_ary) > 1) {
-            if ($show_more_cc == false) {
-                if ($i == 1) {
-                    /* From a search... */
-                    $bcc_string .= '&nbsp;(<A HREF="' . $base_uri .
-                                   "src/read_body.php?mailbox=$urlMailbox&amp;passed_id=$passed_id&amp;";
-                    if (isset($where) && isset($what)) {
-                        $bcc_string .= 'what=' . urlencode($what)."&amp;where=".urlencode($where)."&amp;show_more_cc=1&amp;show_more=$show_more\">$echo_more</A>)";
-                    } else {
-                        $bcc_string .= "sort=$sort&amp;startMessage=$startMessage&amp;show_more_cc=1&amp;show_more=$show_more\">$echo_more</A>)";
-                    }
-                    $i = count($bcc_ary);
-                }
-            } else if ($i == 1) {
-                /* From a search... */
-                $bcc_string .= '&nbsp;(<A HREF="' . $base_uri .
-                               "src/read_body.php?mailbox=$urlMailbox&amp;passed_id=$passed_id&amp;";
-                if (isset($where) && isset($what)) {
-                    $bcc_string .= 'what=' . urlencode($what)."&amp;where=".urlencode($where)."&amp;show_more_cc=0&amp;show_more=$show_more\">$echo_less</A>)";
-                } else {
-                    $bcc_string .= "sort=$sort&amp;startMessage=$startMessage&amp;show_more_cc=0&amp;show_more=$show_more\">$echo_less</A>)";
-                }
-            }
-        }
-    }
-}
-else {
-    $bcc_string = '';
-}
+$bcc = formatRecipientString($message->header->bcc, "bcc");
+$bcc_string = $bcc['str'];
+$url_bcc_string = $bcc['url_str'];
 
 if ($default_use_priority) {
     $priority_level = substr($message->header->priority,0,1);
@@ -572,6 +644,22 @@ if ($default_use_priority) {
 /** make sure everything will display in HTML format **/
 $from_name = decodeHeader(htmlspecialchars($message->header->from));
 $subject = decodeHeader(htmlspecialchars($message->header->subject));
+$identity = '';
+$idents = getPref($data_dir, $username, 'identities');
+if (!empty($idents) && $idents > 1) {
+    for ($i = 1; $i < $idents; $i++) {
+        $enc_from_name = '"'. 
+            encodeHeader(getPref($data_dir, 
+                                 $username, 
+                                 'full_name' . $i)) .
+            '" <' . getPref($data_dir, $username, 
+                            'email_address' . $i) . '>';
+        if (htmlspecialchars($enc_from_name) == $from_name) {
+            $identity = $i;
+            break;
+        }
+    }
+}
 
 do_hook('read_body_top');
 echo '<BR>' .
@@ -584,10 +672,10 @@ echo '<BR>' .
      '<A HREF="' . $base_uri . 'src/';
 
 if ($where && $what) {
-    if( $pos == '' ) {
-        $pos = 0;
+    if ($pos == '') {
+        $pos=0;
     }
-    echo "search.php?where$pos=".urlencode($where)."&amp;pos=$pos&amp;what$pos=".urlencode($what)."&amp;mailbox=$urlMailbox\">";
+    echo "search.php?where=".urlencode($where)."&amp;pos=$pos&amp;what=".urlencode($what)."&amp;mailbox=$urlMailbox\">";
 } else {
     echo "right_main.php?sort=$sort&amp;startMessage=$startMessage&amp;mailbox=$urlMailbox\">";
 }
@@ -601,14 +689,35 @@ if ($where && $what) {
 }
 echo _("Delete") . '</A>&nbsp;';
 if (($mailbox == $draft_folder) && ($save_as_draft)) {
-    echo '|&nbsp;<A HREF="' . $base_uri .
-         "src/compose.php?mailbox=$mailbox&amp;send_to=$to_string&amp;send_to_cc=$cc_string&amp;send_to_bcc=$bcc_string&amp;subject=$url_subj&amp;draft_id=$passed_id&amp;ent_num=$ent_num\"";
+    $comp_uri = $base_uri . "src/compose.php?mailbox=$mailbox&amp;".
+                "identity=$identity&amp;send_to=$url_to_string&amp;".
+		"send_to_cc=$url_cc_string&amp;send_to_bcc=$url_bcc_string&amp;".
+		"subject=$url_subj&amp;mailprio=$priority_level&amp;".
+		"draft_id=$passed_id&amp;ent_num=$ent_num";
+    
     if ($compose_new_win == '1') {
-        echo 'TARGET="compose_window" onClick="comp_in_new()"';
+        echo "<a href=\"javascript:void(0)\" onclick=\"comp_in_new(false,'$comp_uri')\"";
+    } else {
+        echo '|&nbsp;<A HREF="' . $comp_uri .'"';
     }
     echo '>'.
          _("Resume Draft") . '</a>';
 }
+if ($mailbox == $sent_folder) {
+    $comp_uri = $base_uri . "src/compose.php?mailbox=$mailbox&amp;".
+                "identity=$identity&amp;send_to=$url_to_string&amp;".
+		"send_to_cc=$url_cc_string&amp;send_to_bcc=$url_bcc_string&amp;".
+		"subject=$url_subj&amp;mailprio=$priority_level&amp;".
+		"ent_num=$ent_num&amp;passed_id=$passed_id&amp;edit_as_new=1";
+
+    if ($compose_new_win == '1') {
+        echo "<a href=\"javascript:void(0)\" onclick=\"comp_in_new(false,'$comp_uri')\"";
+    } else {
+        echo '|&nbsp;<A HREF="' . $comp_uri .'"';
+    }
+    echo '>'.
+          _("Edit Message as New") . '</a>';
+}
 
 echo '&nbsp;&nbsp;' .
                    '</SMALL>' .
@@ -617,7 +726,6 @@ echo '&nbsp;&nbsp;' .
                    '<SMALL>';
 
 if ( !($where && $what) ) {
-
     if ($currentArrayIndex == -1) {
         echo 'Previous&nbsp;|&nbsp;Next';
     } else {
@@ -640,31 +748,49 @@ if ( !($where && $what) ) {
 
 echo                '</SMALL>' .
                 '</TD><TD WIDTH="33%" ALIGN="RIGHT">' .
-                   '<SMALL>' .
-                   '<A HREF="' . $base_uri . "src/compose.php?forward_id=$passed_id&amp;forward_subj=$url_subj&amp;".
-                    ($default_use_priority?"mailprio=$priority_level&amp;":'')
-                    ."mailbox=$urlMailbox&amp;ent_num=$ent_num\"";
-    if ($compose_new_win == '1') {
-        echo 'TARGET="compose_window" onClick="comp_in_new()"';
-    }
+                   '<SMALL>' ;
+
+$comp_uri = $base_uri . "src/compose.php?forward_id=$passed_id&amp;".
+            "forward_subj=$url_subj&amp;".
+	    ($default_use_priority?"mailprio=$priority_level&amp;":'').
+            "mailbox=$urlMailbox&amp;ent_num=$ent_num";
+	     		   
+if ($compose_new_win == '1') {
+    echo "<a href=\"javascript:void(0)\" onclick=\"comp_in_new(false,'$comp_uri')\"";
+} else {
+    echo '|&nbsp;<A HREF="' . $comp_uri .'"';
+}
+
     echo '>'.
     _("Forward") .
-    '</A>&nbsp;|&nbsp;' .
-                   '<A HREF="' . $base_uri . "src/compose.php?send_to=$url_replyto&amp;reply_subj=$url_subj&amp;".
-                    ($default_use_priority?"mailprio=$priority_level&amp;":'').
-                    "reply_id=$passed_id&amp;mailbox=$urlMailbox&amp;ent_num=$ent_num\"";
-    if ($compose_new_win == '1') {
-        echo 'TARGET="compose_window" onClick="comp_in_new()"';
-    }
+    '</A>&nbsp;|&nbsp;';
+
+$comp_uri = $base_uri . "src/compose.php?send_to=$url_replyto&amp;".
+            "reply_subj=$url_subj&amp;".
+            ($default_use_priority?"mailprio=$priority_level&amp;":'').
+            "reply_id=$passed_id&amp;mailbox=$urlMailbox&amp;ent_num=$ent_num";
+	        
+if ($compose_new_win == '1') {
+    echo "<a href=\"javascript:void(0)\" onclick=\"comp_in_new(false,'$comp_uri')\"";
+} else {
+    echo '|&nbsp;<A HREF="' . $comp_uri .'"';
+}
+
     echo '>'.
     _("Reply") .
-    '</A>&nbsp;|&nbsp;' .
-                   '<A HREF="' . $base_uri . "src/compose.php?send_to=$url_replytoall&amp;send_to_cc=$url_replytoallcc&amp;reply_subj=$url_subj&amp;".
-                    ($default_use_priority?"mailprio=$priority_level&amp;":'').
-                    "reply_id=$passed_id&amp;mailbox=$urlMailbox&amp;ent_num=$ent_num\"";
-    if ($compose_new_win == '1') {
-        echo 'TARGET="compose_window" onClick="comp_in_new()"';
-    }
+    '</A>&nbsp;|&nbsp;';
+
+$comp_uri = $base_uri . "src/compose.php?send_to=$url_replytoall&amp;".
+            "send_to_cc=$url_replytoallcc&amp;reply_subj=$url_subj&amp;".
+            ($default_use_priority?"mailprio=$priority_level&amp;":'').
+            "reply_id=$passed_id&amp;mailbox=$urlMailbox&amp;ent_num=$ent_num";
+
+if ($compose_new_win == '1') {
+    echo "<a href=\"javascript:void(0)\" onclick=\"comp_in_new(false,'$comp_uri')\"";
+} else {
+    echo '|&nbsp;<A HREF="' . $comp_uri .'"';
+}
+
     echo '>'.
     _("Reply All") .
     '</A>&nbsp;&nbsp;' .
@@ -710,9 +836,11 @@ echo       '<TR>' .
              '<TD BGCOLOR="' . $color[0] . '" ALIGN="RIGHT">' .
     _("From:") .
              '</TD><TD BGCOLOR="' . $color[0] . '">' .
-                "<B>$from_name</B>&nbsp;\n" .
-             '</TD>' .
+                "<B>$from_name</B>&nbsp;\n";
+                do_hook("read_body_after_from");
+echo             '</TD>' .
           '</TR>';
+
 /** date **/
 echo       '<TR>' . "\n" .
              '<TD BGCOLOR="' . $color[0] . '" ALIGN="RIGHT">' . "\n" .
@@ -793,7 +921,6 @@ if ($default_use_mdn) {
 
         $read = sqimap_run_command ($imapConnection, "FETCH $passed_id FLAGS", true,
                                 $response, $readmessage);
-
         $MDN_flag_present = preg_match( '/.*\$MDNSent/i', $read[0]);
 
         if (trim($MDN_to) &&
@@ -840,7 +967,7 @@ if ($default_use_mdn) {
             // if no MDNsupport don't use the annoying popup messages
             else if (  !$FirstTimeSee ) {
                 $sendreceipt = 'send';
-                $url = "\"read_body.php?mailbox=$mailbox&amp;passed_id=$passed_id&amp;startMessage=$startMessage&amp;show_more=$show_more&amp;sendreceipt=$sendreceipt\"";
+                $url = "\"read_body.php?mailbox=$mailbox&passed_id=$passed_id&startMessage=$startMessage&show_more=$show_more&sendreceipt=$sendreceipt\"";
                 echo       '<TR>' .
                             "<TD BGCOLOR=\"$color[9]\"  ALIGN=RIGHT VALIGN=TOP>" .
                                 _("Read receipt") . ': ' .
@@ -853,15 +980,14 @@ if ($default_use_mdn) {
             }
             else {
                 $sendreceipt = 'send';
-                $url = "\"read_body.php?mailbox=$mailbox&amp;passed_id=$passed_id&amp;startMessage=$startMessage&amp;show_more=$show_more&amp;sendreceipt=$sendreceipt\"";
+                $url = "\"read_body.php?mailbox=$mailbox&passed_id=$passed_id&startMessage=$startMessage&show_more=$show_more&sendreceipt=$sendreceipt\"";
                 if ($javascript_on) {
                 echo "<script language=\"javascript\" type=\"text/javascript\">  \n" .
                     '<!-- ' . "\n" .
                     "               if (window.confirm(\"" .
                     _("The message sender has requested a response to indicate that you have read this message. Would you like to send a receipt?") .
                     "\")) {  \n" .
-                    "                       window.location=($url); \n" .
-                    '                       window.reload()' . "\n" .
+		    '                       window.open('.$url.',"right");' . "\n". 
                     '               }' . "\n" .
                     '// -->' . "\n" .
                     '</script>' . "\n";
@@ -895,11 +1021,12 @@ if ($default_use_mdn) {
                 if ( SendMDN( $MDN_to, $final_recipient ) > 0 && $supportMDN ) {
                     ToggleMDNflag( true);
                 }
+		ClearAttachments();
             }
             $sendreceipt = 'removeMDN';
             $url = "\"read_body.php?mailbox=$mailbox&amp;passed_id=$passed_id&amp;startMessage=$startMessage&amp;show_more=$show_more&amp;sendreceipt=$sendreceipt\"";
             $sendreceipt='';
-			/*
+            /*
             if ($MDNDebug && $supportMDN) {
             echo "      <TR>\n" .
                     "         <TD BGCOLOR=\"$color[9]\"  ALIGN=RIGHT VALIGN=TOP>\n" .
