@@ -11,7 +11,7 @@
  * sends it to retrievalerror@squirrelmail.org
  * Of course, this only happens when the end user has chosen to do so
  *
- * $Id: retrievalerror.php,v 1.9 2002/02/19 12:31:13 brong Exp $
+ * $Id: retrievalerror.php,v 1.11.2.1 2002/06/21 18:42:17 teepe Exp $
  */
 
 require_once('../src/validate.php');
@@ -20,8 +20,7 @@ require_once('../functions/smtp.php');
 require_once('../functions/page_header.php');
 require_once('../src/load_prefs.php');
 
-$destination = 'retrievalerror@squirrelmail.org';
-
+$destination = 'retrievalerror@sqml.org';
 $attachments = array();
 
 function ClearAttachments() {
@@ -88,7 +87,7 @@ if ($imap_stream) {
 
 $body .= "\nFETCH line for gathering the whole message: $topline2\n";
 
-sendMessage($destination, '', '', 'submitted message', $body, 0);
+sendMessage($destination, '', '', 'submitted message', $body, False, 0);
 
 displayPageHeader($color, $mailbox);
 
