@@ -10,7 +10,7 @@
  * shown can be given as parameters. If the user is not logged in
  * this file will verify username and password.
  *
- * $Id: webmail.php,v 1.67 2002/02/22 07:44:54 kink Exp $
+ * $Id: webmail.php,v 1.73 2002/05/24 16:12:42 stekkel Exp $
  */
 
 require_once('../functions/strings.php');
@@ -21,8 +21,10 @@ require_once('../functions/plugin.php');
 require_once('../functions/i18n.php');
 require_once('../functions/auth.php');
 
-ereg( '(^.*/)[^/]+/[^/]+$', $PHP_SELF, $regs);
-$base_uri = $regs[1];
+if (!function_exists('sqm_baseuri')){
+    require_once('../functions/display_messages.php');
+}
+$base_uri = sqm_baseuri();
 
 session_start();
 is_logged_in();
@@ -59,10 +61,10 @@ if ($left_size == "") {
 }
 
 if ($location_of_bar == 'right') {
-    echo "<FRAMESET COLS=\"*, $left_size\" BORDER=0>";
+    echo "<FRAMESET COLS=\"*, $left_size\" BORDER=0 ID=fs1>";
 }
 else {
-    echo "<FRAMESET COLS=\"$left_size, *\" BORDER=0>";
+    echo "<FRAMESET COLS=\"$left_size, *\" BORDER=0 ID=fs1>";
 }
 
 /*
@@ -90,7 +92,6 @@ if ($right_frame == 'right_main.php') {
 } elseif ($right_frame == 'folders.php') {
     $right_frame_url = 'folders.php';
 } elseif ($right_frame == 'compose.php') {
-    $rcptaddress = urlencode($rcptaddress);
     $right_frame_url = "compose.php?send_to=$rcptaddress";
 } else {
     $right_frame_url = 'right_main.php';
@@ -104,7 +105,7 @@ else {
     echo '<FRAME SRC="left_main.php" NORESIZE NAME="left">'.
          "<FRAME SRC=\"$right_frame_url\" NORESIZE NAME=\"right\">";
 }
-
+do_hook('webmail_bottom');
 ?>
 </FRAMESET>
 </HEAD></HTML>
